@@ -46,7 +46,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 /* Set RAPTOR_DEBUG to > 1 to get lots of buffer related debugging */
@@ -66,9 +66,9 @@ struct raptor_ntriples_parser_context_s {
   /* current line */
   unsigned char *line;
   /* current line length */
-  int line_length;
+  size_t line_length;
   /* current char in line buffer */
-  int offset;
+  size_t offset;
 
   char last_char;
   
@@ -275,7 +275,8 @@ typedef enum {
 
 
 static int 
-raptor_ntriples_term_valid(unsigned char c, int position, 
+raptor_ntriples_term_valid(raptor_parser* rdf_parser,
+                           unsigned char c, int position, 
                            raptor_ntriples_term_class term_class) 
 {
   int result = 0;
@@ -310,7 +311,8 @@ raptor_ntriples_term_valid(unsigned char c, int position,
       break;
       
     default:
-      RAPTOR_FATAL2("Unknown ntriples term %d", term_class);
+      raptor_parser_error(rdf_parser, "Unknown N-Triples term class %d",
+                          term_class);
   }
 
   return result;
@@ -367,6 +369,8 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
   
   /* find end of string, fixing backslashed characters on the way */
   while(*lenp > 0) {
+    int unichar_width;
+
     c = *p;
 
     p++;
@@ -377,9 +381,9 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
     if(allow_utf8) {
       if(c > 0x7f) {
         /* just copy the UTF-8 bytes through */
-        size_t unichar_len;
+        int unichar_len;
         unichar_len = raptor_unicode_utf8_string_get_char(p - 1, 1 + *lenp, NULL);
-        if(unichar_len > *lenp) {
+        if(unichar_len < 0 || RAPTOR_GOOD_CAST(size_t, unichar_len) > *lenp) {
           raptor_parser_error(rdf_parser, "UTF-8 encoding error at character %d (0x%02X) found.", c, c);
           /* UTF-8 encoding had an error or ended in the middle of a string */
           return 1;
@@ -410,7 +414,7 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
         break;
       }
 
-      if(!raptor_ntriples_term_valid(c, position, term_class)) {
+      if(!raptor_ntriples_term_valid(rdf_parser, c, position, term_class)) {
         if(end_char) {
           /* end char was expected, so finding an invalid thing is an error */
           raptor_parser_error(rdf_parser, "Missing terminating '%c' (found '%c')", end_char, c);
@@ -479,8 +483,8 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
 
         p += ulen;
         (*lenp) -= ulen;
-        rdf_parser->locator.column += ulen;
-        rdf_parser->locator.byte += ulen;
+        rdf_parser->locator.column += RAPTOR_GOOD_CAST(int, ulen);
+        rdf_parser->locator.byte += RAPTOR_GOOD_CAST(int, ulen);
         
         if(unichar > raptor_unicode_max_codepoint) {
           raptor_parser_error(rdf_parser,
@@ -489,13 +493,21 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
           break;
         }
 
+        unichar_width = raptor_unicode_utf8_string_put_char(unichar, dest, 4);
+        if(unichar_width < 0) {
+          raptor_parser_error(rdf_parser,
+                              "Illegal Unicode character with code point #x%lX.", 
+                              unichar);
+          break;
+        }
+
         /* The destination length is set here to 4 since we know that in 
          * all cases, the UTF-8 encoded output sequence is always shorter
          * than the input sequence, and the buffer is edited in place.
          *   \uXXXX: 6 bytes input - UTF-8 max 3 bytes output
          *   \uXXXXXXXX: 10 bytes input - UTF-8 max 4 bytes output
          */
-        dest += raptor_unicode_utf8_string_put_char(unichar, dest, 4);
+        dest += (int)unichar_width;
         break;
 
       default:
@@ -526,6 +538,8 @@ raptor_ntriples_term(raptor_parser* rdf_parser,
 }
 
 
+#define MAX_NTRIPLES_TERMS 4
+
 static int
 raptor_ntriples_parse_line(raptor_parser* rdf_parser,
                            unsigned char *buffer, size_t len,
@@ -535,9 +549,11 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
   int i;
   unsigned char *p;
   unsigned char *dest;
-  unsigned char *terms[4];
-  size_t term_lengths[4];
-  raptor_term_type term_types[4];
+  unsigned char *terms[MAX_NTRIPLES_TERMS] = { NULL, NULL, NULL, NULL};
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  size_t term_lengths[MAX_NTRIPLES_TERMS] = {0, 0, 0, 0};
+#endif
+  raptor_term_type term_types[MAX_NTRIPLES_TERMS] = {RAPTOR_TERM_TYPE_UNKNOWN, RAPTOR_TERM_TYPE_UNKNOWN, RAPTOR_TERM_TYPE_UNKNOWN, RAPTOR_TERM_TYPE_UNKNOWN};
   size_t term_length = 0;
   unsigned char *object_literal_language = NULL;
   unsigned char *object_literal_datatype = NULL;
@@ -584,8 +600,8 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
   /* Check for terminating '.' */
   if(p[len-1] != '.') {
     /* Move current location to point to problem */
-    rdf_parser->locator.column += len-2;
-    rdf_parser->locator.byte += len-2;
+    rdf_parser->locator.column += RAPTOR_BAD_CAST(int, len - 2);
+    rdf_parser->locator.byte += RAPTOR_BAD_CAST(int, len - 2);
     raptor_parser_error(rdf_parser, "Missing . at end of line");
     return 0;
   }
@@ -594,10 +610,13 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
   len--;
 
 
-  /* Must be triple */
+  /* Must be triple/quad */
 
   for(i = 0; i < max_terms; i++) {
     if(!len) {
+      /* context is optional in nquads */
+      if (i == 3)
+	break;
       raptor_parser_error(rdf_parser, "Unexpected end of line");
       goto cleanup;
     }
@@ -773,7 +792,10 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
 
 
     /* Store term */
-    terms[i] = dest; term_lengths[i] = term_length;
+    terms[i] = dest;
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+    term_lengths[i] = term_length;
+#endif
 
     /* Whitespace must separate the terms */
     if(i < 2 && !isspace((int)*p)) {
@@ -833,7 +855,7 @@ raptor_ntriples_parse_line(raptor_parser* rdf_parser,
                                      object_literal_datatype,
                                      terms[3], term_types[3]);
 
-  rdf_parser->locator.byte += len;
+  rdf_parser->locator.byte += RAPTOR_BAD_CAST(int, len);
 
  cleanup:
 
@@ -860,7 +882,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
   if(!len)
     return 0;
 
-  buffer = (unsigned char*)RAPTOR_MALLOC(cstring, ntriples_parser->line_length + len + 1);
+  buffer = RAPTOR_MALLOC(unsigned char*, ntriples_parser->line_length + len + 1);
   if(!buffer) {
     raptor_parser_fatal_error(rdf_parser, "Out of memory");
     return 1;
@@ -868,7 +890,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 
   if(ntriples_parser->line_length) {
     memcpy(buffer, ntriples_parser->line, ntriples_parser->line_length);
-    RAPTOR_FREE(cstring, ntriples_parser->line);
+    RAPTOR_FREE(char*, ntriples_parser->line);
   }
 
   ntriples_parser->line = buffer;
@@ -941,7 +963,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #endif
   }
 
-  ntriples_parser->offset = start-buffer;
+  ntriples_parser->offset = start - buffer;
 
   len = ntriples_parser->line_length - ntriples_parser->offset;
     
@@ -951,7 +973,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     RAPTOR_DEBUG3("collapsing buffer from %d to %d bytes\n", ntriples_parser->line_length, (unsigned int)len);
 #endif
-    buffer = (unsigned char*)RAPTOR_MALLOC(cstring, len + 1);
+    buffer = RAPTOR_MALLOC(unsigned char*, len + 1);
     if(!buffer) {
       raptor_parser_fatal_error(rdf_parser, "Out of memory");
       return 1;
@@ -962,7 +984,7 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
            len);
     buffer[len] = '\0';
 
-    RAPTOR_FREE(cstring, ntriples_parser->line);
+    RAPTOR_FREE(char*, ntriples_parser->line);
 
     ntriples_parser->line = buffer;
     ntriples_parser->line_length -= ntriples_parser->offset;
@@ -1008,7 +1030,7 @@ raptor_ntriples_parse_start(raptor_parser* rdf_parser)
 }
 
 
-#ifdef RAPTOR_PARSER_NTRIPLES
+#if defined RAPTOR_PARSER_NTRIPLES || defined RAPTOR_PARSER_NQUADS
 static int
 raptor_ntriples_parse_recognise_syntax(raptor_parser_factory* factory, 
                                        const unsigned char *buffer, size_t len,
@@ -1088,7 +1110,8 @@ raptor_ntriples_parse_recognise_syntax(raptor_parser_factory* factory,
 
 static const char* const ntriples_names[2] = { "ntriples", NULL };
 
-static const char* const ntriples_uri_strings[2] = {
+static const char* const ntriples_uri_strings[3] = {
+  "http://www.w3.org/ns/formats/N-Triples",
   "http://www.w3.org/TR/rdf-testcases/#ntriples",
   NULL
 };
@@ -1144,6 +1167,7 @@ raptor_nquads_parse_recognise_syntax(raptor_parser_factory* factory,
                                      const char *mime_type)
 {
   int score = 0;
+  int ntriples_score;
   
   if(suffix) {
     if(!strcmp((const char*)suffix, "nq"))
@@ -1164,7 +1188,11 @@ raptor_nquads_parse_recognise_syntax(raptor_parser_factory* factory,
       score += 2;
   }
   
-  /* Do not guess using content since it looks so similar to N-Triples */
+  /* ntriples is a subset of nquads, score higher than ntriples */
+  ntriples_score = raptor_ntriples_parse_recognise_syntax(factory, buffer, len, identifier, suffix, mime_type);
+  if(ntriples_score > 0) {
+    score += ntriples_score + 1;
+  }
 
   return score;
 }
@@ -1177,8 +1205,9 @@ static const char* const nquads_uri_strings[2] = {
   NULL
 };
   
-#define NQUADS_TYPES_COUNT 0
+#define NQUADS_TYPES_COUNT 1
 static const raptor_type_q nquads_types[NQUADS_TYPES_COUNT + 1] = {
+  { "text/x-nquads", 13, 10},
   { NULL, 0, 0}
 };
 
