@@ -47,11 +47,11 @@
 
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
 #define RAPTOR_AVLTREE_DEBUG1(msg) RAPTOR_DEBUG1(msg)
 #else
 #define RAPTOR_AVLTREE_DEBUG1(msg)
@@ -126,7 +126,9 @@ static void raptor_avltree_balance_right(raptor_avltree* tree, raptor_avltree_no
 static raptor_avltree_node* raptor_avltree_search_internal(raptor_avltree* tree, raptor_avltree_node* node, const void* p_data);
 static int raptor_avltree_visit_internal(raptor_avltree* tree, raptor_avltree_node* node, int depth, raptor_avltree_visit_handler visit_fn, void* user_data);
 static void raptor_free_avltree_internal(raptor_avltree* tree, raptor_avltree_node* node);
-
+#ifdef RAPTOR_DEBUG
+static void raptor_avltree_check_internal(raptor_avltree* tree, raptor_avltree_node* node, unsigned int* count_p);
+#endif
 
 
 /**
@@ -146,7 +148,7 @@ raptor_new_avltree(raptor_data_compare_handler compare_handler,
 {
   raptor_avltree* tree;
   
-  tree = (raptor_avltree*)RAPTOR_MALLOC(raptor_avltree, sizeof(*tree));
+  tree = RAPTOR_MALLOC(raptor_avltree*, sizeof(*tree));
   if(!tree)
     return NULL;
 
@@ -253,10 +255,15 @@ raptor_avltree_add(raptor_avltree* tree, void* p_data)
 {
   int rebalancing = FALSE;
   int rv;
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  RAPTOR_AVLTREE_DEBUG1("Checking tree before adding\n");
+  raptor_avltree_check(tree);
+#endif
 
   rv = raptor_avltree_sprout(tree, NULL, &tree->root, p_data,
                              &rebalancing);
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  RAPTOR_AVLTREE_DEBUG1("Checking tree after adding\n");
   raptor_avltree_check(tree);
 #endif
 
@@ -282,12 +289,19 @@ raptor_avltree_remove(raptor_avltree* tree, void* p_data)
   int rebalancing = FALSE;
   void* rdata;
   
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  RAPTOR_AVLTREE_DEBUG1("Checking tree before removing\n");
+  raptor_avltree_dump(tree,stderr);
+  raptor_avltree_check(tree);
+#endif
   rdata = raptor_avltree_delete_internal(tree, &tree->root, p_data,
                                          &rebalancing);
   if(rdata)
     tree->size--;
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  RAPTOR_AVLTREE_DEBUG1("Checking tree after removing\n");
+  raptor_avltree_dump(tree,stderr);
   raptor_avltree_check(tree);
 #endif
   
@@ -399,6 +413,33 @@ raptor_avltree_check_node(raptor_avltree* tree, raptor_avltree_node* node,
       abort();
     }
   }
+
+  if(node->left) {
+    if(node->left->parent != node) {
+      if(fn && where)
+        fprintf(stderr, "%s (%s): ", fn, where);
+      fputs("ERROR left child node ", stderr);
+      raptor_avltree_print_node(node->left);
+      fputs(" has no reference to this[parent] node ", stderr);
+      raptor_avltree_print_node(node);
+      fputc('\n', stderr);
+      fflush(stderr);
+      abort();
+    }
+  }
+  if(node->right) {
+    if(node->right->parent != node) {
+      if(fn && where)
+        fprintf(stderr, "%s (%s): ", fn, where);
+      fputs("ERROR right child node ", stderr);
+      raptor_avltree_print_node(node->right);
+      fputs(" has no reference to this[parent] node ", stderr);
+      raptor_avltree_print_node(node);
+      fputc('\n', stderr);
+      fflush(stderr);
+      abort();
+    }
+  }
 }
 #endif
 
@@ -441,6 +482,10 @@ raptor_avltree_sprout_left(raptor_avltree* tree, raptor_avltree_node** node_pp,
     case -1:
       /* left branch was already too long. rebalance */
       RAPTOR_AVLTREE_DEBUG1("LESS: case -1: rebalancing\n");
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+      RAPTOR_AVLTREE_DEBUG1("Tree before rebalancing\n");
+      raptor_avltree_dump(tree, stderr);
+#endif
       p1 = (*node_pp)->left;
 
       if(p1->balance == -1) {
@@ -482,12 +527,23 @@ raptor_avltree_sprout_left(raptor_avltree* tree, raptor_avltree_node** node_pp,
           p1->balance = -1;
         else
           p1->balance = 0;
+
         *node_pp = p2;
         (*node_pp)->parent = p_parent;
       } /* end else */
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+      RAPTOR_AVLTREE_DEBUG1("Tree after rebalancing\n");
+      raptor_avltree_dump(tree, stderr);
+#endif
 
       (*node_pp)->balance = 0;
       *rebalancing_p = FALSE;
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+      if(1) {
+        unsigned int discard = 0;
+        raptor_avltree_check_internal(tree, *node_pp, &discard);
+      }
+#endif
   } /* end switch */
 
   return FALSE;
@@ -533,6 +589,10 @@ raptor_avltree_sprout_right(raptor_avltree* tree,
       RAPTOR_AVLTREE_DEBUG1("MORE: balance was off, need to rebalance\n");
       p1 = (*node_pp)->right;
       
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+      RAPTOR_AVLTREE_DEBUG1("Tree before rebalancing\n");
+      raptor_avltree_dump(tree, stderr);
+#endif
       if(p1->balance == 1) {
         /* RR */
         RAPTOR_AVLTREE_DEBUG1("MORE: single RR\n");
@@ -578,8 +638,18 @@ raptor_avltree_sprout_right(raptor_avltree* tree,
         (*node_pp)->parent = p_parent;
       } /* end else */
       
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+      RAPTOR_AVLTREE_DEBUG1("Tree after rebalancing\n");
+      raptor_avltree_dump(tree, stderr);
+#endif
       (*node_pp)->balance = 0;
       *rebalancing_p = FALSE;
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+      if(1) {
+        unsigned int discard = 0;
+        raptor_avltree_check_internal(tree, *node_pp, &discard);
+      }
+#endif
   } /* end switch */
 
   return FALSE;
@@ -600,18 +670,30 @@ raptor_avltree_sprout(raptor_avltree* tree, raptor_avltree_node* parent,
 {
   int cmp;
 
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_AVLTREE_DEBUG1("Enter\n");
+  if ( *node_pp) { 
+      raptor_avltree_print_node(*node_pp);
+      RAPTOR_AVLTREE_DEBUG1("\n");
+  }
+  else {
+      RAPTOR_AVLTREE_DEBUG1("Nil node\n");
+  }
+#endif
 
   /* If grounded, add the node here, set the rebalance flag and return */
   if(!*node_pp) {
     RAPTOR_AVLTREE_DEBUG1("grounded. adding new node, setting rebalancing flag true\n");
-    *node_pp = (raptor_avltree_node*)RAPTOR_MALLOC(raptor_avltree_node,
-                                                   sizeof(**node_pp));
+    *node_pp = RAPTOR_MALLOC(raptor_avltree_node*, sizeof(**node_pp));
     if(!*node_pp) {
       if(tree->free_handler)
         tree->free_handler(p_data);
       return RAPTOR_AVLTREE_ENOMEM;
     }
+
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+    RAPTOR_DEBUG2("Creating new node %p\n", *node_pp);
+#endif
     
     (*node_pp)->parent = parent;
     (*node_pp)->left = NULL;
@@ -621,10 +703,21 @@ raptor_avltree_sprout(raptor_avltree* tree, raptor_avltree_node* parent,
     *rebalancing_p = TRUE;
 
     tree->size++;
+
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+    raptor_avltree_check_node(tree, *node_pp, 0, 0);
+
+    RAPTOR_AVLTREE_DEBUG1("Tree now looks this way\n");
+    raptor_avltree_dump(tree,stderr);
+#endif
     
     return FALSE;
   }
 
+  /* check node */
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  raptor_avltree_check_node(tree, *node_pp, 0, 0);
+#endif
   /* compare the data */
   cmp = tree->compare_handler(p_data, (*node_pp)->data);
   if(cmp < 0)
@@ -696,10 +789,14 @@ raptor_avltree_delete_internal(raptor_avltree* tree,
     if(pr_q->right == NULL) {
       RAPTOR_AVLTREE_DEBUG1("right subtree null\n");
       *node_pp = pr_q->left;
+      if(*node_pp)
+        (*node_pp)->parent = pr_q->parent;
       *rebalancing_p = TRUE;
     } else if(pr_q->left == NULL) {
       RAPTOR_AVLTREE_DEBUG1("right subtree non-null, left subtree null\n");
       *node_pp = pr_q->right;
+      if(*node_pp)
+        (*node_pp)->parent = pr_q->parent;
       *rebalancing_p = TRUE;
     } else {
       RAPTOR_AVLTREE_DEBUG1("neither subtree null\n");
@@ -735,11 +832,15 @@ raptor_avltree_delete_internal2(raptor_avltree* tree,
       raptor_avltree_balance_right(tree, ppr_r, rebalancing_p);
 
   } else {
+    raptor_avltree_node* ppr_r_left_new_parent;
     rdata = (*ppr_q)->data;
 
     (*ppr_q)->data = (*ppr_r)->data;
     *ppr_q = *ppr_r;
+    ppr_r_left_new_parent = (*ppr_r)->parent;
     *ppr_r = (*ppr_r)->left;
+    if(*ppr_r)
+      (*ppr_r)->parent = ppr_r_left_new_parent;
     *rebalancing_p = TRUE;
   }
 
@@ -1184,8 +1285,7 @@ raptor_new_avltree_iterator(raptor_avltree* tree, void* range,
 {
   raptor_avltree_iterator* iterator;
 
-  iterator = (raptor_avltree_iterator*)RAPTOR_CALLOC(raptor_avltree_iterator,
-                                                     1, sizeof(*iterator));
+  iterator = RAPTOR_CALLOC(raptor_avltree_iterator*, 1, sizeof(*iterator));
   if(!iterator)
     return NULL;
 
@@ -1470,7 +1570,7 @@ typedef struct
   int failed;
 } visit_state;
   
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
 static int
 print_string(int depth, void* data, void *user_data) 
 {
@@ -1540,7 +1640,7 @@ main(int argc, char *argv[])
     int rc;
     void* node;
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     fprintf(stderr, "%s: Adding tree item '%s'\n", program, items[i]);
 #endif
   
@@ -1565,7 +1665,7 @@ main(int argc, char *argv[])
     }
   }
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Printing tree\n", program);
   vs.fh = stderr;
   vs.count = 0;
@@ -1580,7 +1680,7 @@ main(int argc, char *argv[])
   for(i = 0; delete_items[i]; i++) {
     int rc;
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     fprintf(stderr, "%s: Deleting tree item '%s'\n", program, delete_items[i]);
 #endif
   
@@ -1598,7 +1698,7 @@ main(int argc, char *argv[])
   }
 
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Walking tree forwards via iterator\n", program);
 #endif
   iter = raptor_new_avltree_iterator(tree, NULL, NULL, 1);
@@ -1610,7 +1710,7 @@ main(int argc, char *argv[])
               i, result, data);
       exit(1);
     }
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     fprintf(stderr, "%3d: Got '%s'\n", i, data);
 #endif
     if(raptor_avltree_iterator_next(iter))
@@ -1623,7 +1723,7 @@ main(int argc, char *argv[])
   raptor_free_avltree_iterator(iter);
 
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Checking tree\n", program);
 #endif
   vs.count = 0;
@@ -1652,7 +1752,7 @@ main(int argc, char *argv[])
   }
   
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Freeing tree\n", program);
 #endif
   raptor_free_avltree(tree);
