@@ -58,15 +58,15 @@ void* raptor_sign_calloc(size_t nmemb, size_t size);
 void* raptor_sign_realloc(void *ptr, size_t size);
 void raptor_sign_free(void *ptr);
   
-#define RAPTOR_MALLOC(type, size)   raptor_sign_malloc(size)
-#define RAPTOR_CALLOC(type, nmemb, size) raptor_sign_calloc(nmemb, size)
-#define RAPTOR_REALLOC(type, ptr, size) raptor_sign_realloc(ptr, size)
-#define RAPTOR_FREE(type, ptr)   raptor_sign_free(ptr)
+#define RAPTOR_MALLOC(type, size)   (type)raptor_sign_malloc(size)
+#define RAPTOR_CALLOC(type, nmemb, size) (type)raptor_sign_calloc(nmemb, size)
+#define RAPTOR_REALLOC(type, ptr, size) (type)raptor_sign_realloc(ptr, size)
+#define RAPTOR_FREE(type, ptr)   raptor_sign_free((void*)ptr)
 
 #else
-#define RAPTOR_MALLOC(type, size) malloc(size)
-#define RAPTOR_CALLOC(type, nmemb, size) calloc(nmemb, size)
-#define RAPTOR_REALLOC(type, ptr, size) realloc(ptr, size)
+#define RAPTOR_MALLOC(type, size) (type)malloc(size)
+#define RAPTOR_CALLOC(type, nmemb, size) (type)calloc(nmemb, size)
+#define RAPTOR_REALLOC(type, ptr, size) (type)realloc(ptr, size)
 #define RAPTOR_FREE(type, ptr)   free((void*)ptr)
 
 #endif
@@ -166,17 +166,6 @@ void raptor_sign_free(void *ptr);
 #define MAX_ASCII_INT_SIZE 13
   
 /* XML parser includes */
-#ifdef RAPTOR_XML_EXPAT
-#ifdef HAVE_EXPAT_H
-#include <expat.h>
-#endif
-#ifdef HAVE_XMLPARSE_H
-#include <xmlparse.h>
-#endif
-#endif
-
-
-
 
 #ifdef RAPTOR_XML_LIBXML
 
@@ -204,21 +193,6 @@ extern void raptor_libxml_update_document_locator(raptor_sax2* sax2, raptor_loca
 #endif
 
 
-/* expat-only prototypes */
-
-#ifdef RAPTOR_XML_EXPAT
-/* raptor_expat.c exports */
-extern void raptor_expat_init(raptor_sax2* sax2, raptor_uri *base_uri);
-extern void raptor_expat_update_document_locator(raptor_sax2* sax2, raptor_locator *locator);
-
-/* raptor_parse.c */
-void raptor_xml_unparsed_entity_decl_handler(void *user_data, const unsigned char* entityName, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId, const unsigned char* notationName);
-int raptor_xml_external_entity_ref_handler(void *user_data, const unsigned char* context, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId);
-
-/* end of expat-only */
-#endif
-
-
 typedef struct raptor_parser_factory_s raptor_parser_factory;
 typedef struct raptor_serializer_factory_s raptor_serializer_factory;
 typedef struct raptor_id_set_s raptor_id_set;
@@ -447,7 +421,7 @@ struct raptor_namespace_s {
   /* NULL means is the default namespace */
   const unsigned char *prefix;
   /* needed to safely compare prefixed-names */
-  int prefix_length;
+  unsigned int prefix_length;
   /* URI of namespace or NULL for default */
   raptor_uri *uri;
   /* parsing depth that this ns was added.  It will
@@ -469,6 +443,14 @@ raptor_namespace** raptor_namespace_stack_to_array(raptor_namespace_stack *nstac
 #endif
 
 
+/* Size of buffer to use when reading from a file */
+#ifdef BUFSIZ
+#define RAPTOR_READ_BUFFER_SIZE BUFSIZ
+#else
+#define RAPTOR_READ_BUFFER_SIZE 4096
+#endif
+
+
 /*
  * Raptor parser object
  */
@@ -534,6 +516,9 @@ struct raptor_parser_s {
 
   /* internal data for lexers */
   void* lexer_user_data;
+
+  /* internal read buffer */
+  unsigned char buffer[RAPTOR_READ_BUFFER_SIZE + 1];
 };
 
 
@@ -586,6 +571,9 @@ struct raptor_parser_factory_s {
 
   /* get the current graph (OPTIONAL) - if not implemented, the current graph is always the default (NULL) and start/end graph marks are synthesised */
   raptor_uri* (*get_graph)(raptor_parser* rdf_parser);
+
+  /* get the locator (OPTIONAL) */
+  raptor_locator* (*get_locator)(raptor_parser* rdf_parser);
 };
 
 
@@ -699,7 +687,7 @@ int raptor_parser_set_uri_filter_no_net(void *user_data, raptor_uri* uri);
 void raptor_parser_parse_uri_write_bytes(raptor_www* www, void *userdata, const void *ptr, size_t size, size_t nmemb);
 void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 void raptor_parser_error(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
-void raptor_parser_error_varargs(raptor_parser* parser, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(2, 0);
+void raptor_parser_log_error_varargs(raptor_parser* parser, raptor_log_level level, const char *message, va_list arguments) RAPTOR_PRINTF_FORMAT(3, 0);
 void raptor_parser_warning(raptor_parser* parser, const char *message, ...) RAPTOR_PRINTF_FORMAT(2, 3);
 
 /* logging */
@@ -713,7 +701,7 @@ void raptor_log_error(raptor_world* world, raptor_log_level level, raptor_locato
 
 typedef struct raptor_rdfxml_parser_s raptor_rdfxml_parser;
 
-/* Prototypes for common expat/libxml parsing event-handling functions */
+/* Prototypes for common libxml parsing event-handling functions */
 extern void raptor_xml_start_element_handler(void *user_data, const unsigned char *name, const unsigned char **atts);
 extern void raptor_xml_end_element_handler(void *user_data, const unsigned char *name);
 /* s is not 0 terminated. */
@@ -721,7 +709,7 @@ extern void raptor_xml_characters_handler(void *user_data, const unsigned char *
 extern void raptor_xml_cdata_handler(void *user_data, const unsigned char *s, int len);
 void raptor_xml_comment_handler(void *user_data, const unsigned char *s);
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
 void raptor_rdfxml_parser_stats_print(raptor_rdfxml_parser* rdf_xml_parser, FILE *stream);
 #endif
 
@@ -746,8 +734,8 @@ int raptor_check_ordinal(const unsigned char *name);
 #endif
 
 
-/* raptor_nfc.c */
-int raptor_nfc_check (const unsigned char* string, size_t len, int *error);
+/* raptor_nfc_icu.c */
+int raptor_nfc_icu_check (const unsigned char* string, size_t len, int *error);
 
 
 /* raptor_namespace.c */
@@ -776,7 +764,7 @@ struct raptor_qname_s {
   raptor_uri *uri;
   /* optional value - used when name is an attribute */
   const unsigned char *value;
-  unsigned int value_length;
+  size_t value_length;
 };
 
 
@@ -809,6 +797,76 @@ int raptor_init_parser_nquads(raptor_world* world);
 
 void raptor_terminate_parser_grddl_common(raptor_world *world);
 
+#ifdef RAPTOR_PARSER_RDFA
+#define rdfa_add_item raptor_librdfa_rdfa_add_item
+#define rdfa_append_to_list_mapping raptor_librdfa_rdfa_append_to_list_mapping
+#define rdfa_canonicalize_string raptor_librdfa_rdfa_canonicalize_string
+#define rdfa_complete_current_property_value_triples raptor_librdfa_rdfa_complete_current_property_value_triples
+#define rdfa_complete_incomplete_triples raptor_librdfa_rdfa_complete_incomplete_triples
+#define rdfa_complete_list_triples raptor_librdfa_rdfa_complete_list_triples
+#define rdfa_complete_object_literal_triples raptor_librdfa_rdfa_complete_object_literal_triples
+#define rdfa_complete_relrev_triples raptor_librdfa_rdfa_complete_relrev_triples
+#define rdfa_complete_type_triples raptor_librdfa_rdfa_complete_type_triples
+#define rdfa_copy_list raptor_librdfa_rdfa_copy_list
+#define rdfa_copy_mapping raptor_librdfa_rdfa_copy_mapping
+#define rdfa_create_bnode raptor_librdfa_rdfa_create_bnode
+#define rdfa_create_context raptor_librdfa_rdfa_create_context
+#define rdfa_create_list raptor_librdfa_rdfa_create_list
+#define rdfa_create_list_mapping raptor_librdfa_rdfa_create_list_mapping
+#define rdfa_create_mapping raptor_librdfa_rdfa_create_mapping
+#define rdfa_create_new_element_context raptor_librdfa_rdfa_create_new_element_context
+#define rdfa_create_triple raptor_librdfa_rdfa_create_triple
+#define rdfa_establish_new_1_0_subject raptor_librdfa_rdfa_establish_new_1_0_subject
+#define rdfa_establish_new_1_0_subject_with_relrev raptor_librdfa_ablish_new_1_0_subject_with_relrev
+#define rdfa_establish_new_1_1_subject raptor_librdfa_ablish_new_1_1_subject
+#define rdfa_establish_new_1_1_subject_with_relrev raptor_librdfa_ablish_new_1_1_subject_with_relrev
+#define rdfa_establish_new_inlist_triples raptor_librdfa_ablish_new_inlist_triples
+#define rdfa_free_context raptor_librdfa_free_context
+#define rdfa_free_context_stack raptor_librdfa_free_context_stack
+#define rdfa_free_list raptor_librdfa_rdfa_free_list
+#define rdfa_free_mapping raptor_librdfa_rdfa_free_mapping
+#define rdfa_free_triple raptor_librdfa_rdfa_free_triple
+#define rdfa_get_buffer raptor_librdfa_rdfa_get_buffer
+#define rdfa_get_curie_type raptor_librdfa_rdfa_get_curie_type
+#define rdfa_get_list_mapping raptor_librdfa_rdfa_get_list_mapping
+#define rdfa_get_mapping raptor_librdfa_rdfa_get_mapping
+#define rdfa_init_base raptor_librdfa_rdfa_init_base
+#define rdfa_init_context raptor_librdfa_rdfa_init_context
+#define rdfa_iri_get_base raptor_librdfa_rdfa_iri_get_base
+#define rdfa_join_string raptor_librdfa_rdfa_join_string
+#define rdfa_n_append_string raptor_librdfa_rdfa_n_append_string
+#define rdfa_names raptor_librdfa_rdfa_names
+#define rdfa_next_mapping raptor_librdfa_rdfa_next_mapping
+#define rdfa_parse raptor_librdfa_rdfa_parse
+#define rdfa_parse_buffer raptor_librdfa_rdfa_parse_buffer
+#define rdfa_parse_chunk raptor_librdfa_rdfa_parse_chunk
+#define rdfa_parse_end raptor_librdfa_rdfa_parse_end
+#define rdfa_parse_start raptor_librdfa_rdfa_parse_start
+#define rdfa_pop_item raptor_librdfa_rdfa_pop_item
+#define rdfa_print_list raptor_librdfa_rdfa_print_list
+#define rdfa_print_mapping raptor_librdfa_rdfa_print_mapping
+#define rdfa_print_string raptor_librdfa_rdfa_print_string
+#define rdfa_print_triple raptor_librdfa_rdfa_print_triple
+#define rdfa_print_triple_list raptor_librdfa_rdfa_print_triple_list
+#define rdfa_push_item raptor_librdfa_rdfa_push_item
+#define rdfa_replace_list raptor_librdfa_rdfa_replace_list
+#define rdfa_replace_string raptor_librdfa_rdfa_replace_string
+#define rdfa_resolve_curie raptor_librdfa_rdfa_resolve_curie
+#define rdfa_resolve_curie_list raptor_librdfa_rdfa_resolve_curie_list
+#define rdfa_resolve_relrev_curie raptor_librdfa_rdfa_resolve_relrev_curie
+#define rdfa_resolve_uri raptor_librdfa_rdfa_resolve_uri
+#define rdfa_save_incomplete_list_triples raptor_librdfa_rdfa_save_incomplete_list_triples
+#define rdfa_save_incomplete_triples raptor_librdfa_rdfa_save_incomplete_triples
+#define rdfa_set_buffer_filler raptor_librdfa_rdfa_set_buffer_filler
+#define rdfa_set_default_graph_triple_handler raptor_librdfa_rdfa_set_default_graph_triple_handler
+#define rdfa_set_processor_graph_triple_handler raptor_librdfa_rdfa_set_processor_graph_triple_handler
+#define rdfa_setup_initial_context raptor_librdfa_rdfa_setup_initial_context
+#define rdfa_update_language raptor_librdfa_rdfa_update_language
+#define rdfa_update_mapping raptor_librdfa_rdfa_update_mapping
+#define rdfa_update_uri_mappings raptor_librdfa_rdfa_update_uri_mappings
+#define rdfa_uri_strings raptor_librdfa_rdfa_uri_strings
+#endif
+
 /* raptor_parse.c */
 int raptor_parsers_init(raptor_world* world);
 void raptor_parsers_finish(raptor_world *world);
@@ -861,7 +919,7 @@ extern const raptor_unichar raptor_unicode_max_codepoint;
 
 int raptor_unicode_is_namestartchar(raptor_unichar c);
 int raptor_unicode_is_namechar(raptor_unichar c);
-int raptor_unicode_check_utf8_nfc_string(const unsigned char *input, size_t length);
+int raptor_unicode_check_utf8_nfc_string(const unsigned char *input, size_t length, int* error);
 
 /* raptor_www*.c */
 #ifdef RAPTOR_WWW_LIBXML
@@ -872,7 +930,6 @@ int raptor_unicode_check_utf8_nfc_string(const unsigned char *input, size_t leng
 
 #ifdef RAPTOR_WWW_LIBCURL
 #include <curl/curl.h>
-#include <curl/types.h>
 #include <curl/easy.h>
 #endif
 
@@ -886,7 +943,7 @@ struct  raptor_www_s {
   raptor_world* world;
   char *type;
   int free_type;
-  int total_bytes;
+  size_t total_bytes;
   int failed;
   int status_code;
 
@@ -901,14 +958,11 @@ struct  raptor_www_s {
 
 #ifdef RAPTOR_WWW_LIBXML
   void *ctxt;
-  char buffer[RAPTOR_WWW_BUFFER_SIZE];
   int is_end;
   void *old_xmlGenericErrorContext;
 #endif
 
-#ifdef RAPTOR_WWW_LIBFETCH
-  char buffer[RAPTOR_WWW_BUFFER_SIZE];
-#endif
+  char buffer[RAPTOR_WWW_BUFFER_SIZE + 1];
 
   char *user_agent;
 
@@ -953,6 +1007,8 @@ void raptor_www_error(raptor_www *www, const char *message, ...) RAPTOR_PRINTF_F
 void raptor_www_curl_init(raptor_www *www);
 void raptor_www_curl_free(raptor_www *www);
 int raptor_www_curl_fetch(raptor_www *www);
+int raptor_www_curl_set_ssl_cert_options(raptor_www* www, const char* cert_filename, const char* cert_type, const char* cert_passphrase);
+int raptor_www_curl_set_ssl_verify_options(raptor_www* www, int verify_peer, int verify_host);
 
 void raptor_www_libfetch_init(raptor_www *www);
 void raptor_www_libfetch_free(raptor_www *www);
@@ -962,7 +1018,7 @@ int raptor_www_libfetch_fetch(raptor_www *www);
 raptor_id_set* raptor_new_id_set(raptor_world* world);
 void raptor_free_id_set(raptor_id_set* set);
 int raptor_id_set_add(raptor_id_set* set, raptor_uri* base_uri, const unsigned char *item, size_t item_len);
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
 void raptor_id_set_stats_print(raptor_id_set* set, FILE *stream);
 #endif
 
@@ -1004,12 +1060,6 @@ struct raptor_sax2_s {
   raptor_world* world;
   void* user_data;
   
-#ifdef RAPTOR_XML_EXPAT
-  XML_Parser xp;
-#ifdef EXPAT_UTF8_BOM_CRASH
-  int tokens_count; /* used to see if trying to get location info is safe */
-#endif
-#endif
 #ifdef RAPTOR_XML_LIBXML
   /* structure holding sax event handlers */
   xmlSAXHandler sax;
@@ -1062,6 +1112,12 @@ struct raptor_sax2_s {
 
   /* sax2 init failed - do not try to do anything with it */
   int failed;
+  
+  /* call SAX2 handlers if non-0 */
+  int enabled;
+
+  void* uri_filter_user_data;
+  raptor_uri_filter_func uri_filter;
 };
 
 int raptor_sax2_init(raptor_world* world);
@@ -1087,7 +1143,7 @@ void raptor_sax2_cdata(void* user_data, const unsigned char *s, int len);
 void raptor_sax2_comment(void* user_data, const unsigned char *s);
 void raptor_sax2_unparsed_entity_decl(void* user_data, const unsigned char* entityName, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId, const unsigned char* notationName);
 int raptor_sax2_external_entity_ref(void* user_data, const unsigned char* context, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId);
-
+int raptor_sax2_check_load_uri_string(raptor_sax2* sax2, const unsigned char* uri_string);
 
 /* turtle_parser.y and turtle_lexer.l */
 typedef struct raptor_turtle_parser_s raptor_turtle_parser;
@@ -1122,11 +1178,11 @@ struct raptor_uri_detail_s
 
 
 /* for time_t */
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -1189,7 +1245,6 @@ int raptor_abbrev_node_compare(raptor_abbrev_node* node1, raptor_abbrev_node* no
 int raptor_abbrev_node_equals(raptor_abbrev_node* node1, raptor_abbrev_node* node2);
 raptor_abbrev_node* raptor_abbrev_node_lookup(raptor_avltree* nodes, raptor_term* term);
 
-raptor_abbrev_subject* raptor_new_abbrev_subject(raptor_abbrev_node* node);
 void raptor_free_abbrev_subject(raptor_abbrev_subject* subject);
 int raptor_abbrev_subject_add_property(raptor_abbrev_subject* subject, raptor_abbrev_node* predicate, raptor_abbrev_node* object);
 int raptor_abbrev_subject_compare(raptor_abbrev_subject* subject1, raptor_abbrev_subject* subject2);
@@ -1235,6 +1290,7 @@ int raptor_turtle_writer_set_option(raptor_turtle_writer *turtle_writer, raptor_
 int raptor_turtle_writer_set_option_string(raptor_turtle_writer *turtle_writer, raptor_option option, const unsigned char *value);
 int raptor_turtle_writer_get_option(raptor_turtle_writer *turtle_writer, raptor_option option);
 const unsigned char *raptor_turtle_writer_get_option_string(raptor_turtle_writer *turtle_writer, raptor_option option);
+void raptor_turtle_writer_bnodeid(raptor_turtle_writer* turtle_writer, const unsigned char *bnodeid, size_t len);
 
 
 /**
@@ -1267,7 +1323,7 @@ int raptor_rdfxmla_serialize_set_single_node(raptor_serializer* serializer, rapt
 int raptor_rdfxmla_serialize_set_write_typed_nodes(raptor_serializer* serializer, int value);
 
 /* snprintf.c */
-char* raptor_format_float(char *buffer, size_t *currlen, size_t maxlen, double fvalue, unsigned int min, unsigned int max, int flags);
+size_t raptor_format_integer(char* buffer, size_t bufsize, int integer, unsigned int base, int width, char padding);
 
 /* raptor_world structure */
 #define RAPTOR1_WORLD_MAGIC_1 0
@@ -1359,13 +1415,24 @@ struct raptor_world_s {
 
   int default_generate_bnodeid_handler_base;
   char *default_generate_bnodeid_handler_prefix;
-  size_t default_generate_bnodeid_handler_prefix_length;
+  unsigned int default_generate_bnodeid_handler_prefix_length;
 };
 
 /* raptor_www.c */
 int raptor_www_init(raptor_world* world);
 void raptor_www_finish(raptor_world* world);
 
+
+
+#define RAPTOR_LANG_LEN_FROM_INT(len) (int)(len)
+#define RAPTOR_LANG_LEN_TO_SIZE_T(len) (size_t)(len)
+
+/* Safe casts: widening a value */
+#define RAPTOR_GOOD_CAST(t, v) (t)(v)
+
+/* Unsafe casts: narrowing a value */
+#define RAPTOR_BAD_CAST(t, v) (t)(v)
+
 /* end of RAPTOR_INTERNAL */
 #endif
 
