@@ -47,7 +47,7 @@
 #include <yajl/yajl_parse.h>
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
@@ -80,11 +80,21 @@ typedef enum {
 } raptor_json_term_attrib;
 
 
+/* When YAJL V1 support is dropped, this can be removed */
+#ifdef HAVE_YAJL2
+#define RAPTOR_YAJL_LEN_TYPE size_t
+#else
+#define RAPTOR_YAJL_LEN_TYPE unsigned int
+#endif
+
 /*
  * JSON parser object
  */
 struct raptor_json_parser_context_s {
+#ifdef HAVE_YAJL2
+#else
   yajl_parser_config config;
+#endif
   yajl_handle handle;
 
   /* Parser state */
@@ -109,11 +119,11 @@ static void
 raptor_json_reset_term(raptor_json_parser_context *context)
 {
   if(context->term_value)
-    RAPTOR_FREE(cstring, context->term_value);
+    RAPTOR_FREE(char*, context->term_value);
   if(context->term_lang)
-    RAPTOR_FREE(cstring, context->term_lang);
+    RAPTOR_FREE(char*, context->term_lang);
   if(context->term_datatype)
-    RAPTOR_FREE(cstring, context->term_datatype);
+    RAPTOR_FREE(char*, context->term_datatype);
 
   context->term_value = NULL;
   context->term_lang = NULL;
@@ -123,9 +133,9 @@ raptor_json_reset_term(raptor_json_parser_context *context)
 }
 
 static unsigned char*
-raptor_json_cstring_from_counted_string(raptor_parser *rdf_parser, const unsigned char* str, unsigned int len)
+raptor_json_cstring_from_counted_string(raptor_parser *rdf_parser, const unsigned char* str, RAPTOR_YAJL_LEN_TYPE len)
 {
-  unsigned char *cstr = (unsigned char*)RAPTOR_MALLOC(cstring, len + 1);
+  unsigned char *cstr = RAPTOR_MALLOC(unsigned char*, len + 1);
   if(!cstr) {
     raptor_parser_fatal_error(rdf_parser, "Out of memory");
     return NULL;
@@ -138,7 +148,7 @@ raptor_json_cstring_from_counted_string(raptor_parser *rdf_parser, const unsigne
 }
 
 static raptor_term*
-raptor_json_new_term_from_counted_string(raptor_parser *rdf_parser, const unsigned char* str, unsigned int len)
+raptor_json_new_term_from_counted_string(raptor_parser *rdf_parser, const unsigned char* str, size_t len)
 {
   raptor_term *term = NULL;
 
@@ -149,9 +159,9 @@ raptor_json_new_term_from_counted_string(raptor_parser *rdf_parser, const unsign
   } else {
     raptor_uri *uri = raptor_new_uri_from_counted_string(rdf_parser->world, str, len);
     if(!uri) {
-      unsigned char* cstr = raptor_json_cstring_from_counted_string(rdf_parser, str, len);
+      unsigned char* cstr = raptor_json_cstring_from_counted_string(rdf_parser, str, RAPTOR_BAD_CAST(int, len));
       raptor_parser_error(rdf_parser, "Could not create uri from '%s'", cstr);
-      RAPTOR_FREE(cstring, cstr);
+      RAPTOR_FREE(char*, cstr);
       return NULL;
     }
 
@@ -226,7 +236,13 @@ static int raptor_json_yajl_boolean(void * ctx, int b)
   return 0;
 }
 
-static int raptor_json_yajl_integer(void * ctx, long l)
+#ifdef HAVE_YAJL2
+#define YAJL_INTEGER_CALLBACK_ARG_TYPE long long
+#else
+#define YAJL_INTEGER_CALLBACK_ARG_TYPE long
+#endif
+static int raptor_json_yajl_integer(void * ctx,
+                                    YAJL_INTEGER_CALLBACK_ARG_TYPE integerVal)
 {
   raptor_parser* rdf_parser = (raptor_parser*)ctx;
   raptor_parser_error(rdf_parser, "Integers are not valid in RDF/JSON");
@@ -241,7 +257,7 @@ static int raptor_json_yajl_double(void * ctx, double d)
 }
 
 static int raptor_json_yajl_string(void * ctx, const unsigned char * str,
-                           unsigned int len)
+                                   RAPTOR_YAJL_LEN_TYPE len)
 {
   raptor_parser* rdf_parser = (raptor_parser*)ctx;
   raptor_json_parser_context *context;
@@ -267,7 +283,7 @@ static int raptor_json_yajl_string(void * ctx, const unsigned char * str,
           unsigned char * cstr = raptor_json_cstring_from_counted_string(rdf_parser, str, len);
           context->term_type = RAPTOR_TERM_TYPE_UNKNOWN;
           raptor_parser_error(rdf_parser, "Unknown term type: %s", cstr);
-          RAPTOR_FREE(cstring, cstr);
+          RAPTOR_FREE(char*, cstr);
         }
       break;
       case RAPTOR_JSON_ATTRIB_DATATYPE:
@@ -286,7 +302,7 @@ static int raptor_json_yajl_string(void * ctx, const unsigned char * str,
 }
 
 static int raptor_json_yajl_map_key(void * ctx, const unsigned char * str,
-                            unsigned int len)
+                                    RAPTOR_YAJL_LEN_TYPE len)
 {
   raptor_parser* rdf_parser = (raptor_parser*)ctx;
   raptor_json_parser_context *context;
@@ -504,21 +520,21 @@ static int raptor_json_yajl_end_array(void * ctx)
 
 
 static void*
-raptor_json_yajl_malloc(void *ctx, unsigned int sz)
+raptor_json_yajl_malloc(void *ctx, RAPTOR_YAJL_LEN_TYPE sz)
 {
-  return RAPTOR_MALLOC(cstring, sz);
+  return RAPTOR_MALLOC(void*, sz);
 }
 
 static void*
-raptor_json_yajl_realloc(void *ctx, void * ptr, unsigned int sz)
+raptor_json_yajl_realloc(void *ctx, void * ptr, RAPTOR_YAJL_LEN_TYPE sz)
 {
-  return RAPTOR_REALLOC(cstring, ptr, sz);
+  return RAPTOR_REALLOC(void*, ptr, sz);
 }
 
 static void
 raptor_json_yajl_free(void *ctx, void * ptr)
 {
-  RAPTOR_FREE(cstring, ptr);
+  RAPTOR_FREE(char*, ptr);
 }
 
 static yajl_alloc_funcs raptor_json_yajl_alloc_funcs = {
@@ -562,8 +578,11 @@ raptor_json_parse_init(raptor_parser* rdf_parser, const char *name)
   raptor_statement_init(&context->statement, rdf_parser->world);
 
   /* Configure the parser */
+#ifdef HAVE_YAJL2
+#else
   context->config.allowComments = 1;
   context->config.checkUTF8 = 0;
+#endif
 
   return 0;
 }
@@ -591,8 +610,8 @@ raptor_json_parse_terminate(raptor_parser* rdf_parser)
 
 static int
 raptor_json_parse_chunk(raptor_parser* rdf_parser,
-                            const unsigned char *s, size_t len,
-                            int is_end)
+                        const unsigned char *s, size_t len,
+                        int is_end)
 {
   raptor_json_parser_context *context = (raptor_json_parser_context*)rdf_parser->context;
   yajl_status status;
@@ -600,12 +619,16 @@ raptor_json_parse_chunk(raptor_parser* rdf_parser,
 
   if(len) {
     /* Parse the chunk passed to us */
-    status = yajl_parse(context->handle, s, len);
+    status = yajl_parse(context->handle, s, RAPTOR_BAD_CAST(int, len));
 
-    if(status != yajl_status_ok &&
-        status != yajl_status_insufficient_data)
+    if(status != yajl_status_ok 
+#ifdef HAVE_YAJL2
+#else
+       && status != yajl_status_insufficient_data
+#endif
+    )
     {
-      unsigned char * str = yajl_get_error(context->handle, 1, s, len);
+      unsigned char * str = yajl_get_error(context->handle, 1, s, RAPTOR_BAD_CAST(int, len));
       raptor_parser_error(rdf_parser, "YAJL error: %s", (const char *) str);
       result = 1;
       yajl_free_error(context->handle, str);
@@ -614,7 +637,11 @@ raptor_json_parse_chunk(raptor_parser* rdf_parser,
 
   if(is_end) {
     /* parse any remaining buffered data */
-    status = yajl_parse_complete(context->handle);
+#ifdef HAVE_YAJL2
+#else
+#define yajl_complete_parse(h) yajl_parse_complete(h)
+#endif
+    status = yajl_complete_parse(context->handle);
 
     if(status != yajl_status_ok)
     {
@@ -644,7 +671,10 @@ raptor_json_parse_start(raptor_parser* rdf_parser)
   /* Initialise a new parser */
   context->handle = yajl_alloc(
     &raptor_json_yajl_callbacks,
+#ifdef HAVE_YAJL2
+#else
     &context->config,
+#endif
     &raptor_json_yajl_alloc_funcs,
     (void *)rdf_parser
   );
@@ -655,6 +685,12 @@ raptor_json_parse_start(raptor_parser* rdf_parser)
   }
 
   /* Initialise the parse state */
+#ifdef HAVE_YAJL2
+  yajl_config(context->handle, yajl_allow_comments, 1);
+  yajl_config(context->handle, yajl_dont_validate_strings, 1);
+#else
+#endif
+
   context->state = RAPTOR_JSON_STATE_ROOT;
   raptor_json_reset_term(context);
   raptor_statement_clear(&context->statement);
@@ -704,7 +740,7 @@ raptor_json_parse_recognise_syntax(raptor_parser_factory* factory,
 static const char* const json_names[2] = { "json", NULL };
 
 static const char* const json_uri_strings[2] = {
-  "http://n2.talis.com/wiki/RDF_JSON_Specification",
+  "http://docs.api.talis.com/platform-api/output-types/rdf-json",
   NULL
 };
   
