@@ -43,7 +43,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
@@ -145,16 +145,117 @@ raptor_libxml_hasExternalSubset (void* user_data)
 
 static xmlParserInputPtr
 raptor_libxml_resolveEntity(void* user_data, 
-                            const xmlChar *publicId, const xmlChar *systemId) {
+                            const xmlChar *publicId, const xmlChar *systemId)
+{
   raptor_sax2* sax2 = (raptor_sax2*)user_data;
-  return libxml2_resolveEntity(sax2->xc, publicId, systemId);
+  xmlParserCtxtPtr ctxt = sax2->xc;
+  const unsigned char *uri_string = NULL;
+  xmlParserInputPtr entity_input = NULL;
+  int load_entity = 0;
+
+  if(ctxt->input)
+    uri_string = RAPTOR_GOOD_CAST(const unsigned char *, ctxt->input->filename);
+
+  if(!uri_string)
+    uri_string = RAPTOR_GOOD_CAST(const unsigned char *, ctxt->directory);
+
+  load_entity = RAPTOR_OPTIONS_GET_NUMERIC(sax2, RAPTOR_OPTION_LOAD_EXTERNAL_ENTITIES);
+  if(load_entity)
+    load_entity = raptor_sax2_check_load_uri_string(sax2, uri_string);
+
+  if(load_entity) {
+    entity_input = xmlLoadExternalEntity(RAPTOR_GOOD_CAST(const char*, uri_string),
+                                         RAPTOR_GOOD_CAST(const char*, publicId),
+                                         ctxt);
+  } else {
+    RAPTOR_DEBUG4("Not loading entity URI %s by policy for publicId '%s' systemId '%s'\n", uri_string, publicId, systemId);
+  }
+  
+  return entity_input;
 }
 
 
 static xmlEntityPtr
-raptor_libxml_getEntity(void* user_data, const xmlChar *name) {
+raptor_libxml_getEntity(void* user_data, const xmlChar *name)
+{
   raptor_sax2* sax2 = (raptor_sax2*)user_data;
-  return libxml2_getEntity(sax2->xc, name);
+  xmlParserCtxtPtr xc = sax2->xc;
+  xmlEntityPtr ret = NULL;
+
+  if(!xc)
+    return NULL;
+
+  if(!xc->inSubset) {
+    /* looks for hardcoded set of entity names - lt, gt etc. */
+    ret = xmlGetPredefinedEntity(name);
+    if(ret) {
+      RAPTOR_DEBUG2("Entity '%s' found in predefined set\n", name);
+      return ret;
+    }
+  }
+
+  /* This section uses xmlGetDocEntity which looks for entities in
+   * memory only, never from a file or URI 
+   */
+  if(xc->myDoc && (xc->myDoc->standalone == 1)) {
+    RAPTOR_DEBUG2("Entity '%s' document is standalone\n", name);
+    /* Document is standalone: no entities are required to interpret doc */
+    if(xc->inSubset == 2) {
+      xc->myDoc->standalone = 0;
+      ret = xmlGetDocEntity(xc->myDoc, name);
+      xc->myDoc->standalone = 1;
+    } else {
+      ret = xmlGetDocEntity(xc->myDoc, name);
+      if(!ret) {
+        xc->myDoc->standalone = 0;
+        ret = xmlGetDocEntity(xc->myDoc, name);
+        xc->myDoc->standalone = 1;
+      }
+    }
+  } else {
+    ret = xmlGetDocEntity(xc->myDoc, name);
+  }
+
+  if(ret && !ret->children &&
+    (ret->etype == XML_EXTERNAL_GENERAL_PARSED_ENTITY)) {
+    /* Entity is an external general parsed entity. It may be in a
+     * catalog file, user file or user URI
+     */
+    int val = 0;
+    xmlNodePtr children;
+    int load_entity = 0;
+
+    load_entity = RAPTOR_OPTIONS_GET_NUMERIC(sax2, RAPTOR_OPTION_LOAD_EXTERNAL_ENTITIES);
+    if(load_entity)
+      load_entity = raptor_sax2_check_load_uri_string(sax2, ret->URI);
+
+    if(!load_entity) {
+      RAPTOR_DEBUG2("Not getting entity URI %s by policy\n", ret->URI);
+      children = xmlNewText((const xmlChar*)"");
+    } else {
+      /* Disable SAX2 handlers so that the SAX2 events do not all get
+       * sent to callbacks during dealing with the entity parsing.
+       */
+      sax2->enabled = 0;
+      val = xmlParseCtxtExternalEntity(xc, ret->URI, ret->ExternalID, &children);
+      sax2->enabled = 1;
+    }
+    
+    if(!val) {
+      xmlAddChildList((xmlNodePtr)ret, children);
+    } else {
+      xc->validate = 0;
+      return NULL;
+    }
+    
+    ret->owner = 1;
+
+    /* Mark this entity as having been checked - never do this again */
+    if(!ret->checked)
+      ret->checked = 1;
+  }
+
+  return ret;
 }
 
 
@@ -247,10 +348,10 @@ raptor_libxml_warning(void* user_data, const char *msg, ...)
 {
   raptor_sax2* sax2 = NULL;
   va_list args;
-  int prefix_length = strlen(xml_warning_prefix);
+  int prefix_length = RAPTOR_BAD_CAST(int, strlen(xml_warning_prefix));
   int length;
   char *nmsg;
-  size_t msg_len;
+  int msg_len;
 
   /* Work around libxml2 bug - sometimes the sax2->error
    * returns a ctx, sometimes the userdata
@@ -265,9 +366,9 @@ raptor_libxml_warning(void* user_data, const char *msg, ...)
 
   raptor_libxml_update_document_locator(sax2, sax2->locator);
 
-  msg_len = strlen(msg);
+  msg_len = RAPTOR_BAD_CAST(int, strlen(msg));
   length = prefix_length + msg_len + 1;
-  nmsg = (char*)RAPTOR_MALLOC(cstring, length);
+  nmsg = RAPTOR_MALLOC(char*, length);
   if(nmsg) {
     memcpy(nmsg, xml_warning_prefix, prefix_length); /* Do not copy NUL */
     memcpy(nmsg + prefix_length, msg, msg_len + 1); /* Copy NUL */
@@ -281,7 +382,7 @@ raptor_libxml_warning(void* user_data, const char *msg, ...)
                            nmsg ? nmsg : msg, 
                            args);
   if(nmsg)
-    RAPTOR_FREE(cstring,nmsg);
+    RAPTOR_FREE(char*, nmsg);
   va_end(args);
 }
 
@@ -291,11 +392,13 @@ raptor_libxml_error_common(void* user_data, const char *msg, va_list args,
                            const char *prefix, int is_fatal)
 {
   raptor_sax2* sax2 = NULL;
-  int prefix_length = strlen(prefix);
+  int prefix_length = RAPTOR_BAD_CAST(int, strlen(prefix));
   int length;
   char *nmsg;
-  size_t msg_len;
-  
+  int msg_len;
+  raptor_world* world = NULL;
+  raptor_locator* locator = NULL;
+
   if(user_data) {
     /* Work around libxml2 bug - sometimes the sax2->error
      * returns a user_data, sometimes the userdata
@@ -307,12 +410,17 @@ raptor_libxml_error_common(void* user_data, const char *msg, va_list args,
       sax2 = (raptor_sax2*)((xmlParserCtxtPtr)user_data)->userData;
   }
 
-  if(sax2->locator)
-    raptor_libxml_update_document_locator(sax2, sax2->locator);
+  if(sax2) {
+    world = sax2->world;
+    locator = sax2->locator;
+    
+    if(locator)
+      raptor_libxml_update_document_locator(sax2, sax2->locator);
+  }
 
-  msg_len = strlen(msg);
+  msg_len = RAPTOR_BAD_CAST(int, strlen(msg));
   length = prefix_length + msg_len + 1;
-  nmsg = (char*)RAPTOR_MALLOC(cstring, length);
+  nmsg = RAPTOR_MALLOC(char*, length);
   if(nmsg) {
     memcpy(nmsg, prefix, prefix_length); /* Do not copy NUL */
     memcpy(nmsg + prefix_length, msg, msg_len + 1); /* Copy NUL */
@@ -321,20 +429,20 @@ raptor_libxml_error_common(void* user_data, const char *msg, va_list args,
   }
 
   if(is_fatal)
-    raptor_log_error_varargs(sax2->world,
+    raptor_log_error_varargs(world,
                              RAPTOR_LOG_LEVEL_FATAL,
-                             sax2->locator, 
+                             locator, 
                              nmsg ? nmsg : msg, 
                              args);
   else
-    raptor_log_error_varargs(sax2->world,
+    raptor_log_error_varargs(world,
                              RAPTOR_LOG_LEVEL_ERROR,
-                             sax2->locator, 
+                             locator, 
                              nmsg ? nmsg : msg, 
                              args);
   
   if(nmsg)
-    RAPTOR_FREE(cstring,nmsg);
+    RAPTOR_FREE(char*, nmsg);
 }
 
 
@@ -356,16 +464,16 @@ raptor_libxml_generic_error(void* user_data, const char *msg, ...)
   raptor_world* world = (raptor_world*)user_data;
   va_list args;
   const char* prefix = xml_generic_error_prefix;
-  int prefix_length = strlen(prefix);
+  int prefix_length = RAPTOR_BAD_CAST(int, strlen(prefix));
   int length;
   char *nmsg;
-  size_t msg_len;
+  int msg_len;
   
   va_start(args, msg);
 
-  msg_len = strlen(msg);
+  msg_len = RAPTOR_BAD_CAST(int, strlen(msg));
   length = prefix_length + msg_len + 1;
-  nmsg = (char*)RAPTOR_MALLOC(cstring, length);
+  nmsg = RAPTOR_MALLOC(char*, length);
   if(nmsg) {
     memcpy(nmsg, prefix, prefix_length); /* Do not copy NUL */
     memcpy(nmsg + prefix_length, msg, msg_len + 1); /* Copy NUL */
@@ -379,7 +487,7 @@ raptor_libxml_generic_error(void* user_data, const char *msg, ...)
                            args);
   
   if(nmsg)
-    RAPTOR_FREE(cstring,nmsg);
+    RAPTOR_FREE(char*, nmsg);
 
   va_end(args);
 }
@@ -413,18 +521,18 @@ raptor_libxml_validation_warning(void* user_data, const char *msg, ...)
 {
   va_list args;
   raptor_sax2* sax2 = (raptor_sax2*)user_data;
-  int prefix_length = strlen(xml_validation_warning_prefix);
+  int prefix_length = RAPTOR_GOOD_CAST(int, strlen(xml_validation_warning_prefix));
   int length;
   char *nmsg;
-  size_t msg_len;
+  int msg_len;
   
   va_start(args, msg);
 
   raptor_libxml_update_document_locator(sax2, sax2->locator);
 
-  msg_len = strlen(msg);
+  msg_len = RAPTOR_BAD_CAST(int, strlen(msg));
   length = prefix_length + msg_len + 1;
-  nmsg = (char*)RAPTOR_MALLOC(cstring, length);
+  nmsg = RAPTOR_MALLOC(char*, length);
   if(nmsg) {
     memcpy(nmsg, xml_validation_warning_prefix, prefix_length); /* Do not copy NUL */
     memcpy(nmsg + prefix_length, msg, msg_len + 1); /* Copy NUL */
@@ -438,7 +546,7 @@ raptor_libxml_validation_warning(void* user_data, const char *msg, ...)
                            nmsg ? nmsg : msg, 
                            args);
   if(nmsg)
-    RAPTOR_FREE(cstring,nmsg);
+    RAPTOR_FREE(char*, nmsg);
   va_end(args);
 }
 
@@ -745,7 +853,7 @@ raptor_libxml_xmlStructuredError_handler_parsing(void *user_data,
 
   /* user_data may point to a raptor_sax2* */
   if(user_data) {
-    sax2 = (raptor_sax2*)sax2;
+    sax2 = (raptor_sax2*)user_data;
     if(sax2->magic != RAPTOR_LIBXML_MAGIC)
       sax2 = NULL;
   }
