@@ -52,7 +52,6 @@
 /* Raptor includes */
 #include "raptor2.h"
 #include "raptor_internal.h"
-#include "raptor_nfc.h"
 
 
 #undef RAPTOR_NFC_DECODE_DEBUG
@@ -64,7 +63,7 @@
  * @unicode_string: first char of string
  * @end: last char of unicode_string
  */
-static int
+static size_t
 decode_to_utf8(unsigned char *utf8_string, size_t utf8_string_length,
                const char *unicode_string, const char *end)
 {
@@ -80,6 +79,7 @@ decode_to_utf8(unsigned char *utf8_string, size_t utf8_string_length,
   while(p < end) {
     unsigned long c = 0;
     char *endptr;
+    int unicode_width;
 
     if(*p == ' ') {
       p++;
@@ -94,11 +94,19 @@ decode_to_utf8(unsigned char *utf8_string, size_t utf8_string_length,
 
     p = (const char*)endptr;
     
-    u += raptor_unicode_utf8_string_put_char(c, u, (end-p));
-    
-    if((u-utf8_string) > (int)utf8_string_length) {
+    unichar_width = raptor_unicode_utf8_string_put_char(c, u, (end-p));
+    if(unichar_width < 0) {
       fprintf(stderr,
-              "decode_to_utf8 overwote utf8_string buffer at byte %d\n",
+              "decode_to_utf8 Illegal Unicode character with code point #x%lX.", 
+              unichar);
+      break;
+    }
+
+    u += (size_t)unichar_width;
+    
+    if((u-utf8_string) > RAPTOR_GOOD_CAST(int, utf8_string_length)) {
+      fprintf(stderr,
+              "decode_to_utf8 overwrote utf8_string buffer at byte %ld\n",
               (u-utf8_string));
       abort();
     }
@@ -114,18 +122,19 @@ decode_to_utf8(unsigned char *utf8_string, size_t utf8_string_length,
 
 
 static void
-utf8_print(const unsigned char *input, int length, FILE *stream)
+utf8_print(const unsigned char *input, size_t length, FILE *stream)
 {
-  int i = 0;
+  size_t i = 0;
   
   while(i < length && *input) {
     unsigned long c;
     int size = raptor_unicode_utf8_string_get_char(input, length - i, &c);
     if(size <= 0)
       return;
+
     if(i)
       fputc(' ', stream);
-    fprintf(stream, "U+%04X", (int)c);
+    fprintf(stream, "U+%04X", RAPTOR_GOOD_CAST(int, c));
     input += size;
     i += size;
   }
@@ -195,9 +204,6 @@ main (int argc, char *argv[])
     if(*p == '@' || *p == '#')
       continue;
 
-    if(line != 56)
-      continue;
-    
 
     /* skip column 1 */
     while(*p++ != ';')
@@ -209,10 +215,16 @@ main (int argc, char *argv[])
     while(*p++ != ';')
       ;
 
-    column2_len = decode_to_utf8(column2, UNISTR_SIZE, start, p-1);
+    column2_len = decode_to_utf8(column2, UNISTR_SIZE, start, p-2);
     if(column2_len > max_c2_len)
       max_c2_len = column2_len;
-    
+
+#if 0
+    fprintf(stderr, "UTF8 column 2 (%ld bytes) is: '", column2_len);
+    utf8_print(column2, column2_len, stderr);
+    fputs("'\n", stderr);
+#endif
+
     /* skip column 3 */
     while(*p++ != ';')
       ;
@@ -223,10 +235,16 @@ main (int argc, char *argv[])
     while(*p++ != ';')
       ;
 
-    column4_len = decode_to_utf8(column4, UNISTR_SIZE, start, p-1);
+    column4_len = decode_to_utf8(column4, UNISTR_SIZE, start, p-2);
     if(column4_len > max_c4_len)
       max_c4_len = column4_len;
 
+#if 0
+    fprintf(stderr, "UTF8 column 4 (%ld bytes) is: '", column4_len);
+    utf8_print(column4, column4_len, stderr);
+    fputs("'\n", stderr);
+#endif
+
     if(!raptor_unicode_check_utf8_string(column2, column2_len)) {
       fprintf(stderr, "%s:%d: UTF8 column 2 failed on: '", filename, line);
       utf8_print(column2, column2_len, stderr);
