@@ -50,7 +50,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
@@ -172,8 +172,7 @@ raptor_world_register_parser_factory(raptor_world* world,
 {
   raptor_parser_factory *parser = NULL;
   
-  parser = (raptor_parser_factory*)RAPTOR_CALLOC(raptor_parser_factory, 1,
-                                                 sizeof(*parser));
+  parser = RAPTOR_CALLOC(raptor_parser_factory*, 1, sizeof(*parser));
   if(!parser)
     return NULL;
 
@@ -197,8 +196,7 @@ raptor_world_register_parser_factory(raptor_world* world,
 
 
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
-  RAPTOR_DEBUG3("Registered parser %s with context size %d\n",
-                parser->names[0], parser->context_length);
+  RAPTOR_DEBUG2("Registered parser %s\n", parser->desc.names[0]);
 #endif
 
   return parser;
@@ -328,16 +326,14 @@ raptor_new_parser(raptor_world* world, const char *name)
   if(!factory)
     return NULL;
 
-  rdf_parser = (raptor_parser*)RAPTOR_CALLOC(raptor_parser, 1,
-                                             sizeof(*rdf_parser));
+  rdf_parser = RAPTOR_CALLOC(raptor_parser*, 1, sizeof(*rdf_parser));
   if(!rdf_parser)
     return NULL;
 
   rdf_parser->world = world;
   raptor_statement_init(&rdf_parser->statement, world);
   
-  rdf_parser->context = (char*)RAPTOR_CALLOC(raptor_parser_context, 1,
-                                             factory->context_length);
+  rdf_parser->context = RAPTOR_CALLOC(void*, 1, factory->context_length);
   if(!rdf_parser->context) {
     raptor_free_parser(rdf_parser);
     return NULL;
@@ -355,8 +351,9 @@ raptor_new_parser(raptor_world* world, const char *name)
   
   raptor_object_options_init(&rdf_parser->options, RAPTOR_OPTION_AREA_PARSER);
 
-  /* Initialise default (lax) option values */
-  raptor_parser_set_option(rdf_parser, RAPTOR_OPTION_STRICT, NULL, 0);
+  /* set parsing strictness from default value */
+  raptor_parser_set_strict(rdf_parser, 
+                           RAPTOR_OPTIONS_GET_NUMERIC(rdf_parser, RAPTOR_OPTION_STRICT));
 
   if(factory->init(rdf_parser, name)) {
     raptor_free_parser(rdf_parser);
@@ -506,10 +503,6 @@ raptor_free_parser(raptor_parser* rdf_parser)
 }
 
 
-/* Size of XML buffer to use when reading from a file */
-#define RAPTOR_READ_BUFFER_SIZE 4096
-
-
 /**
  * raptor_parser_parse_file_stream:
  * @rdf_parser: parser
@@ -528,10 +521,8 @@ raptor_parser_parse_file_stream(raptor_parser* rdf_parser,
                                 FILE *stream, const char* filename,
                                 raptor_uri *base_uri)
 {
-  /* Read buffer */
-  unsigned char buffer[RAPTOR_READ_BUFFER_SIZE+1];
   int rc = 0;
-  raptor_locator *locator=&rdf_parser->locator;
+  raptor_locator *locator = &rdf_parser->locator;
 
   if(!stream || !base_uri)
     return 1;
@@ -543,10 +534,10 @@ raptor_parser_parse_file_stream(raptor_parser* rdf_parser,
     return 1;
   
   while(!feof(stream)) {
-    int len = fread(buffer, 1, RAPTOR_READ_BUFFER_SIZE, stream);
+    size_t len = fread(rdf_parser->buffer, 1, RAPTOR_READ_BUFFER_SIZE, stream);
     int is_end = (len < RAPTOR_READ_BUFFER_SIZE);
-    buffer[len] = '\0';
-    rc = raptor_parser_parse_chunk(rdf_parser, buffer, len, is_end);
+    rdf_parser->buffer[len] = '\0';
+    rc = raptor_parser_parse_chunk(rdf_parser, rdf_parser->buffer, len, is_end);
     if(rc || is_end)
       break;
   }
@@ -614,7 +605,7 @@ raptor_parser_parse_file(raptor_parser* rdf_parser, raptor_uri *uri,
   if(uri) {
     if(fh)
       fclose(fh);
-    RAPTOR_FREE(cstring, (void*)filename);
+    RAPTOR_FREE(char*, filename);
   }
   if(free_base_uri)
     raptor_free_uri(base_uri);
@@ -629,7 +620,7 @@ raptor_parser_parse_uri_write_bytes(raptor_www* www,
                                     size_t size, size_t nmemb)
 {
   raptor_parse_bytes_context* rpbc = (raptor_parse_bytes_context*)userdata;
-  int len = size*nmemb;
+  size_t len = size * nmemb;
 
   if(!rpbc->started) {
     raptor_uri* base_uri = rpbc->base_uri;
@@ -729,7 +720,12 @@ raptor_parser_parse_uri_with_connection(raptor_parser* rdf_parser,
   int ret = 0;
   raptor_parse_bytes_context rpbc;
   char* ua = NULL;
-  
+  char* cert_filename = NULL;
+  char* cert_type = NULL;
+  char* cert_passphrase = NULL;
+  int ssl_verify_peer;
+  int ssl_verify_host;
+
   if(connection) {
     if(rdf_parser->www)
       raptor_free_www(rdf_parser->www);
@@ -749,7 +745,7 @@ raptor_parser_parse_uri_with_connection(raptor_parser* rdf_parser,
     accept_h = raptor_parser_get_accept_header(rdf_parser);
     if(accept_h) {
       raptor_www_set_http_accept(rdf_parser->www, accept_h);
-      RAPTOR_FREE(cstring, accept_h);
+      RAPTOR_FREE(char*, accept_h);
     }
   }
 
@@ -780,7 +776,24 @@ raptor_parser_parse_uri_with_connection(raptor_parser* rdf_parser,
   ua = RAPTOR_OPTIONS_GET_STRING(rdf_parser, RAPTOR_OPTION_WWW_HTTP_USER_AGENT);
   if(ua)
     raptor_www_set_user_agent(rdf_parser->www, ua);
+
+  cert_filename = RAPTOR_OPTIONS_GET_STRING(rdf_parser,
+                                            RAPTOR_OPTION_WWW_CERT_FILENAME);
+  cert_type = RAPTOR_OPTIONS_GET_STRING(rdf_parser,
+                                        RAPTOR_OPTION_WWW_CERT_TYPE);
+  cert_passphrase = RAPTOR_OPTIONS_GET_STRING(rdf_parser,
+                                              RAPTOR_OPTION_WWW_CERT_PASSPHRASE);
+  if(cert_filename || cert_type || cert_passphrase)
+    raptor_www_set_ssl_cert_options(rdf_parser->www, cert_filename,
+                                    cert_type, cert_passphrase);
   
+  ssl_verify_peer = RAPTOR_OPTIONS_GET_NUMERIC(rdf_parser,
+                                               RAPTOR_OPTION_WWW_SSL_VERIFY_PEER);
+  ssl_verify_host = RAPTOR_OPTIONS_GET_NUMERIC(rdf_parser,
+                                               RAPTOR_OPTION_WWW_SSL_VERIFY_HOST);
+  raptor_www_set_ssl_verify_options(rdf_parser->www, ssl_verify_peer,
+                                    ssl_verify_host);
+
   ret = raptor_www_fetch(rdf_parser->www, uri);
   
   if(!rpbc.started && !ret)
@@ -839,32 +852,35 @@ raptor_parser_error(raptor_parser* parser, const char *message, ...)
 
   va_start(arguments, message);
 
-  raptor_parser_error_varargs(parser, message, arguments);
+  raptor_parser_log_error_varargs(parser, RAPTOR_LOG_LEVEL_ERROR,
+                                  message, arguments);
   
   va_end(arguments);
 }
 
 
 /**
- * raptor_parser_error_varargs:
- * @parser: parser
+ * raptor_parser_log_error_varargs:
+ * @parser: parser (or NULL)
+ * @level: log level
  * @message: error format message
  * @arguments: varargs for message
  *
  * Error from a parser - Internal.
  */  
 void  
-raptor_parser_error_varargs(raptor_parser* parser, const char *message,  
-                            va_list arguments)
+raptor_parser_log_error_varargs(raptor_parser* parser,
+                                raptor_log_level level,
+                                const char *message, va_list arguments)
 {
   if(parser)
     raptor_log_error_varargs(parser->world,
-                             RAPTOR_LOG_LEVEL_ERROR,
+                             level,
                              &parser->locator,
                              message, arguments);
   else
     raptor_log_error_varargs(NULL,
-                             RAPTOR_LOG_LEVEL_ERROR,
+                             level,
                              NULL,
                              message, arguments);
 }
@@ -905,7 +921,10 @@ raptor_parser_warning(raptor_parser* parser, const char *message, ...)
  * @handler: new statement callback function
  *
  * Set the statement handler function for the parser.
- * 
+ *
+ * Use this to set the function to receive statements as the parsing
+ * proceeds. The statement argument to @handler is shared and must be
+ * copied by the caller with raptor_statement_copy().
  **/
 void
 raptor_parser_set_statement_handler(raptor_parser* parser,
@@ -1155,7 +1174,10 @@ raptor_parser_parse_abort(raptor_parser *rdf_parser)
 raptor_locator*
 raptor_parser_get_locator(raptor_parser *rdf_parser)
 {
-  return &rdf_parser->locator;
+  if(rdf_parser->factory->get_locator)
+    return rdf_parser->factory->get_locator(rdf_parser);
+  else
+    return &rdf_parser->locator;
 }
 
 
@@ -1164,8 +1186,8 @@ void
 raptor_stats_print(raptor_parser *rdf_parser, FILE *stream)
 {
 #ifdef RAPTOR_PARSER_RDFXML
-#if RAPTOR_DEBUG > 1
-  if(!strcmp(rdf_parser->factory->name, "rdfxml")) {
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
+  if(!strcmp(rdf_parser->factory->desc.names[0], "rdfxml")) {
     raptor_rdfxml_parser *rdf_xml_parser = (raptor_rdfxml_parser*)rdf_parser->context;
     fputs("raptor parser stats\n  ", stream);
     raptor_rdfxml_parser_stats_print(rdf_xml_parser, stream);
@@ -1187,7 +1209,8 @@ static int
 compare_syntax_score(const void *a, const void *b) {
   return ((struct syntax_score*)b)->score - ((struct syntax_score*)a)->score;
 }
-  
+
+#define RAPTOR_MIN_GUESS_SCORE 2
 
 /**
  * raptor_world_guess_parser_name:
@@ -1203,6 +1226,8 @@ compare_syntax_score(const void *a, const void *b) {
  * Find a parser by scoring recognition of the syntax by a block of
  * characters, the content identifier or a mime type.  The content
  * identifier is typically a filename or URI or some other identifier.
+ *
+ * If the guessing finds only low scores, NULL will be returned.
  * 
  * Return value: a parser name or NULL if no guess could be made
  **/
@@ -1221,9 +1246,9 @@ raptor_world_guess_parser_name(raptor_world* world,
 
   raptor_world_open(world);
 
-  scores = (struct syntax_score*)RAPTOR_CALLOC(syntax_scores,
-                                               raptor_sequence_size(world->parsers),
-                                               sizeof(struct syntax_score));
+  scores = RAPTOR_CALLOC(struct syntax_score*,
+                         raptor_sequence_size(world->parsers),
+                         sizeof(struct syntax_score));
   if(!scores)
     return NULL;
   
@@ -1233,8 +1258,7 @@ raptor_world_guess_parser_name(raptor_world* world,
       unsigned char *from, *to;
 
       p++;
-      suffix = (unsigned char*)RAPTOR_MALLOC(cstring,
-                                             strlen((const char*)p) + 1);
+      suffix = RAPTOR_MALLOC(unsigned char*, strlen((const char*)p) + 1);
       if(!suffix)
         return NULL;
 
@@ -1242,7 +1266,7 @@ raptor_world_guess_parser_name(raptor_world* world,
         unsigned char c = *from++;
         /* discard the suffix if it wasn't '\.[a-zA-Z0-9]+$' */
         if(!isalpha(c) && !isdigit(c)) {
-          RAPTOR_FREE(cstring, suffix);
+          RAPTOR_FREE(char*, suffix);
           suffix = NULL;
           to = NULL;
           break;
@@ -1273,7 +1297,7 @@ raptor_world_guess_parser_name(raptor_world* world,
       if(type_q)
         score = type_q->q;
     }
-    /* mime type match has high Q - return result */
+    /* mime type match has high Q - return factory as result */
     if(score >= 10)
       break;
     
@@ -1289,7 +1313,7 @@ raptor_world_guess_parser_name(raptor_world* world,
           break;
       }
       if(factory_uri_string)
-        /* got an exact match syntax for URI - return result */
+        /* got an exact match syntax for URI - return factory as result */
         break;
     }
     
@@ -1315,20 +1339,21 @@ raptor_world_guess_parser_name(raptor_world* world,
 
     scores[i].score = score < 10 ? score : 10; 
     scores[i].factory = factory;
-#if RAPTOR_DEBUG > 2
-    RAPTOR_DEBUG3("Score %15s : %d\n", factory->name, score);
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 2
+    RAPTOR_DEBUG3("Score %15s : %d\n", factory->desc.names[0], score);
 #endif
   }
   
   if(!factory) {
-    /* sort the scores and pick a factory */
+    /* sort the scores and pick a factory if score is good enough */
     qsort(scores, i, sizeof(struct syntax_score), compare_syntax_score);
-    if(scores[0].score >= 0)
+
+    if(scores[0].score >= RAPTOR_MIN_GUESS_SCORE)
       factory = scores[0].factory;
   }
 
   if(suffix)
-    RAPTOR_FREE(cstring, suffix);
+    RAPTOR_FREE(char*, suffix);
 
   RAPTOR_FREE(syntax_scores, scores);
   
@@ -1445,7 +1470,7 @@ raptor_parser_get_accept_header(raptor_parser* rdf_parser)
   
   /* 9 = strlen("\*\/\*;q=0.1") */
 #define ACCEPT_HEADER_LEN 9
-  accept_header = (char*)RAPTOR_MALLOC(cstring, len + ACCEPT_HEADER_LEN + 1);
+  accept_header = RAPTOR_MALLOC(char*, len + ACCEPT_HEADER_LEN + 1);
   if(!accept_header)
     return NULL;
 
@@ -1501,7 +1526,7 @@ raptor_parser_get_accept_header_all(raptor_world* world)
   
   /* 9 = strlen("\*\/\*;q=0.1") */
 #define ACCEPT_HEADER_LEN 9
-  accept_header = (char*)RAPTOR_MALLOC(cstring, len + ACCEPT_HEADER_LEN + 1);
+  accept_header = RAPTOR_MALLOC(char*, len + ACCEPT_HEADER_LEN + 1);
   if(!accept_header)
     return NULL;
 
@@ -1558,7 +1583,7 @@ raptor_parser_get_content(raptor_parser* rdf_parser, size_t* length_p)
     return NULL;
   
   len = raptor_stringbuffer_length(rdf_parser->sb);
-  buffer = (unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  buffer = RAPTOR_MALLOC(unsigned char*, len + 1);
   if(!buffer)
     return NULL;
 
@@ -1654,8 +1679,6 @@ int
 raptor_parser_parse_iostream(raptor_parser* rdf_parser, raptor_iostream *iostr,
                              raptor_uri *base_uri)
 {
-  /* Read buffer */
-  unsigned char buffer[RAPTOR_READ_BUFFER_SIZE + 1];
   int rc = 0;
 
   RAPTOR_ASSERT_OBJECT_POINTER_RETURN_VALUE(rdf_parser, raptor_parser, 1);
@@ -1666,11 +1689,11 @@ raptor_parser_parse_iostream(raptor_parser* rdf_parser, raptor_iostream *iostr,
     return rc;
   
   while(!raptor_iostream_read_eof(iostr)) {
-    int len = raptor_iostream_read_bytes(buffer, 1, RAPTOR_READ_BUFFER_SIZE,
-                                         iostr);
+    size_t len = raptor_iostream_read_bytes(rdf_parser->buffer, 1,
+                                            RAPTOR_READ_BUFFER_SIZE, iostr);
     int is_end = (len < RAPTOR_READ_BUFFER_SIZE);
 
-    rc = raptor_parser_parse_chunk(rdf_parser, buffer, len, is_end);
+    rc = raptor_parser_parse_chunk(rdf_parser, rdf_parser->buffer, len, is_end);
     if(rc || is_end)
       break;
   }
@@ -1701,7 +1724,7 @@ main(int argc, char *argv[])
   if(!world || raptor_world_open(world))
     exit(1);
   
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Known options:\n", program);
 #endif
 
@@ -1715,7 +1738,7 @@ main(int argc, char *argv[])
     if(!od)
       continue;
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     fprintf(stderr, " %2d %-20s %s <%s>\n", i, od->name, od->label,
             (od->uri ? (const char*)raptor_uri_as_string(od->uri) : ""));
 #endif
@@ -1730,7 +1753,7 @@ main(int argc, char *argv[])
   }
 
   s = raptor_parser_get_accept_header_all(world);
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "Default HTTP accept header: '%s'\n", s);
 #endif
   if(!s) {
@@ -1738,7 +1761,7 @@ main(int argc, char *argv[])
             program);
     return 1;
   }
-  RAPTOR_FREE(cstring, s);
+  RAPTOR_FREE(char*, s);
 
   raptor_free_world(world);
   
