@@ -69,9 +69,7 @@ raptor_new_uri_detail(const unsigned char *uri_string)
   /* The extra +5 is for the 5 \0s that may be added for each component 
    * even if the entire URI is empty 
    */
-  ud = (raptor_uri_detail*)RAPTOR_CALLOC(raptor_uri_detail,
-                                       sizeof(*ud) + uri_len + 5 + 1,
-                                       1);
+  ud = RAPTOR_CALLOC(raptor_uri_detail*, 1, sizeof(*ud) + uri_len + 5 + 1);
   if(!ud)
     return NULL;
   ud->uri_len = uri_len;
@@ -205,7 +203,7 @@ raptor_uri_detail_to_string(raptor_uri_detail *ud, size_t* len_p)
   if(len_p)
     *len_p=len;
   
-  buffer = (unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  buffer = RAPTOR_MALLOC(unsigned char*, len + 1);
   if(!buffer)
     return NULL;
 
@@ -271,7 +269,7 @@ raptor_uri_resolve_uri_reference(const unsigned char *base_uri,
   unsigned char last_char;
   size_t result_len = 0;
   
-#if RAPTOR_DEBUG > 2
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 2
   RAPTOR_DEBUG4("base uri='%s', reference_uri='%s, buffer size %d\n",
                 (base_uri ? (const char*)base_uri : "NULL"),
                 (reference_uri ? (const char*)reference_uri : "NULL"),
@@ -372,30 +370,35 @@ raptor_uri_resolve_uri_reference(const unsigned char *base_uri,
     result.path_len += ref->path_len;
 
   /* the resulting path can be no longer than result.path_len */
-  path_buffer = (unsigned char*)RAPTOR_MALLOC(cstring, result.path_len+1);
+  path_buffer = RAPTOR_MALLOC(unsigned char*, result.path_len + 1);
   if(!path_buffer) {
     result_len = 0;
     goto resolve_tidy;
   }
   result.path = path_buffer;
   *path_buffer = '\0';
-  
-  for(p = base->path + base->path_len-1; p > base->path && *p != '/'; p--)
-    ;
 
-  if(p >= base->path) {
-    result.path_len = p-base->path+1;
-
-    /* Found a /, copy everything before that to path_buffer */
+  if(!ref->path) {
+    /* If there is no reference path, copy the full base over */
+    result.path_len = base->path_len;
     memcpy(path_buffer, base->path, result.path_len);
-    path_buffer[result.path_len]='\0';
-  }
+  } else {
+    /** Otherwise copy base path up to previous / and append ref path */
+    for(p = base->path + base->path_len - 1; p > base->path && *p != '/'; p--)
+      ;
+
+    if(p >= base->path) {
+      result.path_len = p-base->path + 1;
+
+      /* Found a /, copy everything before that to path_buffer */
+      memcpy(path_buffer, base->path, result.path_len);
+      path_buffer[result.path_len] = '\0';
+    }
 
-  if(ref->path) {
     memcpy(path_buffer + result.path_len, ref->path, ref->path_len + 1);
     result.path_len += ref->path_len;
-    path_buffer[result.path_len]='\0';
   }
+  path_buffer[result.path_len] = '\0';
 
 
   /* remove all "./" path components */
@@ -599,7 +602,7 @@ raptor_uri_resolve_uri_reference(const unsigned char *base_uri,
 
   resolve_tidy:
   if(path_buffer)
-    RAPTOR_FREE(cstring, path_buffer);
+    RAPTOR_FREE(char*, path_buffer);
   if(base)
     raptor_free_uri_detail(base);
   if(ref)
@@ -644,7 +647,7 @@ check_resolve(const char *base_uri, const char *reference_uri,
               buffer, result_uri);
       return 1;
   }
-#if RAPTOR_DEBUG > 2
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 2
   fprintf(stderr,
           "%s: raptor_uri_resolve_uri_reference(%s, %s) OK giving '%s'\n",
           program, base_uri, reference_uri, 
@@ -663,7 +666,7 @@ check_parses(const char *uri_string) {
               program, uri_string);
       return 1;
   }
-#if RAPTOR_DEBUG > 2
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 2
   fprintf(stderr, "%s: raptor_new_uri_detail(%s) OK\n",
           program, uri_string);
 #endif
@@ -680,7 +683,7 @@ main(int argc, char *argv[])
 
   program = raptor_basename(argv[0]);
   
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   fprintf(stderr, "%s: Using base URI '%s'\n", program, base_uri);
 #endif
 
@@ -700,7 +703,7 @@ main(int argc, char *argv[])
   failures += check_resolve(base_uri, "gpath/", "http://example.org/bpath/cpath/gpath/");
   failures += check_resolve(base_uri, "/gpath", "http://example.org/gpath");
   failures += check_resolve(base_uri, "//gpath", "http://gpath");
-  failures += check_resolve(base_uri, "?y", "http://example.org/bpath/cpath/?y");
+  failures += check_resolve(base_uri, "?y", "http://example.org/bpath/cpath/d;p?y");
   failures += check_resolve(base_uri, "gpath?y", "http://example.org/bpath/cpath/gpath?y");
   failures += check_resolve(base_uri, "#s", "http://example.org/bpath/cpath/d;p?querystr#s");
   failures += check_resolve(base_uri, "gpath#s", "http://example.org/bpath/cpath/gpath#s");
@@ -782,6 +785,11 @@ main(int argc, char *argv[])
   /* Issue#000177 http://bugs.librdf.org/mantis/view.php?id=177 */
   failures += check_resolve("foo:1234", "9999", "foo:9999");
 
+  /* RDFa 1.1 test 0114 */
+  failures += check_resolve("http://example.org/file",
+                            "?foo=bar../baz",
+                            "http://example.org/file?foo=bar../baz");
+
   return failures;
 }
 
