@@ -43,7 +43,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
@@ -153,8 +153,7 @@ raptor_serializer_register_factory(raptor_world* world,
 {
   raptor_serializer_factory *serializer;
   
-  serializer = (raptor_serializer_factory*)RAPTOR_CALLOC(raptor_serializer_factory, 1,
-                                                         sizeof(*serializer));
+  serializer = RAPTOR_CALLOC(raptor_serializer_factory*, 1, sizeof(*serializer));
   if(!serializer)
     return NULL;
 
@@ -176,8 +175,7 @@ raptor_serializer_register_factory(raptor_world* world,
   }
 
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
-  RAPTOR_DEBUG3("Registered serializer %s with context size %d\n",
-                serializer->names[0], serializer->context_length);
+  RAPTOR_DEBUG2("Registered serializer %s\n", serializer->desc.names[0]);
 #endif
 
   return serializer;
@@ -311,15 +309,13 @@ raptor_new_serializer(raptor_world* world, const char *name)
   if(!factory)
     return NULL;
 
-  rdf_serializer = (raptor_serializer*)RAPTOR_CALLOC(raptor_serializer, 1,
-                                                     sizeof(*rdf_serializer));
+  rdf_serializer = RAPTOR_CALLOC(raptor_serializer*, 1, sizeof(*rdf_serializer));
   if(!rdf_serializer)
     return NULL;
 
   rdf_serializer->world = world;
   
-  rdf_serializer->context = (char*)RAPTOR_CALLOC(raptor_serializer_context, 1,
-                                                 factory->context_length);
+  rdf_serializer->context = RAPTOR_CALLOC(void*, 1, factory->context_length);
   if(!rdf_serializer->context) {
     raptor_free_serializer(rdf_serializer);
     return NULL;
@@ -330,21 +326,6 @@ raptor_new_serializer(raptor_world* world, const char *name)
   raptor_object_options_init(&rdf_serializer->options,
                              RAPTOR_OPTION_AREA_SERIALIZER);
 
-  /* Default options (that are not 0 or NULL) */
-  /* Emit @base directive or equivalent */
-  RAPTOR_OPTIONS_SET_NUMERIC(rdf_serializer, RAPTOR_OPTION_WRITE_BASE_URI, 1);
-  
-  /* Emit relative URIs where possible */
-  RAPTOR_OPTIONS_SET_NUMERIC(rdf_serializer, RAPTOR_OPTION_RELATIVE_URIS, 1);
-
-  /* XML 1.0 output */
-  RAPTOR_OPTIONS_SET_NUMERIC(rdf_serializer,
-                             RAPTOR_OPTION_WRITER_XML_VERSION, 10);
-
-  /* Write XML declaration */
-  RAPTOR_OPTIONS_SET_NUMERIC(rdf_serializer,
-                             RAPTOR_OPTION_WRITER_XML_DECLARATION, 1);
-
   if(factory->init(rdf_serializer, name)) {
     raptor_free_serializer(rdf_serializer);
     return NULL;
@@ -419,7 +400,7 @@ raptor_serializer_start_to_filename(raptor_serializer *rdf_serializer,
   rdf_serializer->locator.uri = rdf_serializer->base_uri;
   rdf_serializer->locator.line = rdf_serializer->locator.column = 0;
 
-  RAPTOR_FREE(cstring, uri_string);
+  RAPTOR_FREE(char*, uri_string);
 
   rdf_serializer->iostream = raptor_new_iostream_to_filename(rdf_serializer->world,
                                                              filename);
