@@ -42,7 +42,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 /*
@@ -257,7 +257,9 @@ raptor_dot_serializer_write_term(raptor_serializer * serializer,
       
     case RAPTOR_TERM_TYPE_UNKNOWN:
     default:
-      RAPTOR_FATAL2("Unknown type %d", term->type);
+      raptor_log_error_formatted(serializer->world, RAPTOR_LOG_LEVEL_ERROR,
+                                 NULL, "Triple has unsupported term type %d",
+                                 term->type);
   }
 }
 
