@@ -42,7 +42,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 #include "raptor_rss.h"
 
@@ -164,8 +164,7 @@ raptor_rss10_set_item_group(raptor_rss10_serializer_context *rss_serializer,
   if(raptor_rss10_get_group_item(rss_serializer, term))
     return 0;
  
-  gm = (raptor_rss_group_map*)RAPTOR_CALLOC(raptor_rss_group_map, 1,
-                                            sizeof(*gm));
+  gm = RAPTOR_CALLOC(raptor_rss_group_map*, 1, sizeof(*gm));
   gm->world = rss_serializer->world;
   gm->term = raptor_term_copy(term);
   gm->item = item;
@@ -731,7 +730,7 @@ raptor_rss10_store_statement(raptor_rss10_serializer_context *rss_serializer,
 
         RAPTOR_DEBUG1("Adding field\n");
         raptor_rss_item_add_field(item, f, field);
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
         RAPTOR_DEBUG2("Stored statement under typed node %p\n", item);
 #endif
 
@@ -749,7 +748,7 @@ raptor_rss10_store_statement(raptor_rss10_serializer_context *rss_serializer,
     if(t) {
       raptor_sequence_push(rss_serializer->triples, t);
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
       fprintf(stderr,"Stored statement: ");
       raptor_statement_print_as_ntriples(s, stderr);
       fprintf(stderr,"\n");
@@ -810,7 +809,7 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
   rss_serializer = (raptor_rss10_serializer_context*)serializer->context;
   rss_model = &rss_serializer->model;
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   if(1) {
     RAPTOR_DEBUG1("Processing statement\n  ");
     raptor_statement_print_as_ntriples(statement, stderr);
@@ -858,7 +857,7 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
           ts = raptor_term_to_string(statement->subject);
           RAPTOR_DEBUG4("Found typed node %i - %s with term %s\n", type,
                         raptor_rss_items_info[type].name, ts);
-          RAPTOR_FREE(cstring, ts);
+          RAPTOR_FREE(char*, ts);
         }
 #endif
         break;
@@ -872,7 +871,7 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
       unsigned char* ts;
       ts = raptor_term_to_string(statement->object);
       RAPTOR_DEBUG2("UNKNOWN typed node with type term %s\n", ts);
-      RAPTOR_FREE(cstring, ts);
+      RAPTOR_FREE(char*, ts);
     }
 #endif
     goto savetriple;
@@ -897,7 +896,7 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
         ts = raptor_term_to_string(statement->subject);
           
         RAPTOR_DEBUG2("RSS item term %s is not in sequence of items\n", ts);
-        RAPTOR_FREE(cstring, ts);
+        RAPTOR_FREE(char*, ts);
       }
 #endif
       item = NULL;
@@ -918,7 +917,7 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
         unsigned char* ts;
         ts = raptor_term_to_string(statement->subject);
         RAPTOR_DEBUG2("Add new enclosure to sequence with term %s\n", ts);
-        RAPTOR_FREE(cstring, ts);
+        RAPTOR_FREE(char*, ts);
       }
 #endif
       
@@ -943,9 +942,14 @@ raptor_rss10_serialize_statement(raptor_serializer* serializer,
 
 
   savetriple:
-  if(!handled)
+  if(!handled) {
     handled = raptor_rss10_store_statement(rss_serializer, statement);
 
+    /* failed to store */
+    if(!handled)
+      return 1;
+  }
+
   return 0;
 }
 
@@ -969,7 +973,7 @@ raptor_rss10_build_items(raptor_rss10_serializer_context *rss_serializer)
     if(!s)
       continue;
     
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
     RAPTOR_DEBUG1("Processing statement\n  ");
     raptor_statement_print_as_ntriples(s, stderr);
     fputc('\n', stderr);
@@ -998,7 +1002,7 @@ raptor_rss10_build_items(raptor_rss10_serializer_context *rss_serializer)
         unsigned char* ts;
         ts = raptor_term_to_string(s->object);
         RAPTOR_DEBUG3("Found RSS 1.0 item %d with term %s\n", ordinal, ts);
-        RAPTOR_FREE(cstring, ts);
+        RAPTOR_FREE(char*, ts);
       }
 #endif
 
@@ -1145,7 +1149,9 @@ raptor_rss10_build_xml_names(raptor_serializer *serializer, int is_entry)
   }
 
 
-  world->rss_fields_info_qnames = (raptor_qname**)RAPTOR_CALLOC(raptor_qname* array, RAPTOR_RSS_FIELDS_SIZE, sizeof(raptor_qname*));
+  world->rss_fields_info_qnames = RAPTOR_CALLOC(raptor_qname**,
+                                                RAPTOR_RSS_FIELDS_SIZE,
+                                                sizeof(raptor_qname*));
   if(!world->rss_fields_info_qnames)
     return;
 
@@ -1162,7 +1168,9 @@ raptor_rss10_build_xml_names(raptor_serializer *serializer, int is_entry)
       return;
   }
 
-  world->rss_types_info_qnames = (raptor_qname**)RAPTOR_CALLOC(raptor_qname* array, RAPTOR_RSS_COMMON_SIZE, sizeof(raptor_qname*));
+  world->rss_types_info_qnames = RAPTOR_CALLOC(raptor_qname**,
+                                               RAPTOR_RSS_COMMON_SIZE,
+                                               sizeof(raptor_qname*));
   if(!world->rss_types_info_qnames)
     return;
   for(i = 0; i < RAPTOR_RSS_COMMON_SIZE; i++) {
@@ -1280,8 +1288,7 @@ raptor_rss10_emit_atom_triples_map(raptor_serializer *serializer, int is_feed,
     at_map_element = raptor_new_xml_element(at_map_qname, NULL, base_uri_copy);
     
     
-    at_map_attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1,
-                                                  sizeof(raptor_qname*));
+    at_map_attrs = RAPTOR_CALLOC(raptor_qname**, 1, sizeof(raptor_qname*));
     ruri_string = raptor_uri_to_relative_uri_string(base_uri,
       serializer->world->rss_fields_info_uris[to_f]);
     at_map_attrs[0] = raptor_new_qname(rss_serializer->nstack, 
@@ -1334,8 +1341,7 @@ raptor_rss10_emit_atom_feed(raptor_serializer *serializer,
   atom_link_element = raptor_new_xml_element(atom_link_qname, NULL,
                                              base_uri_copy);
 
-  atom_link_attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 2, 
-                                                   sizeof(raptor_qname*));
+  atom_link_attrs = RAPTOR_CALLOC(raptor_qname**, 2, sizeof(raptor_qname*));
   ruri_string = raptor_uri_to_relative_uri_string(base_uri, item->uri);
 
   atom_link_attrs[0] = raptor_new_qname(rss_serializer->nstack, 
@@ -1413,7 +1419,7 @@ raptor_rss10_emit_rss_items(raptor_serializer *serializer)
                                                               NULL);
     base_uri_copy = base_uri ? raptor_uri_copy(base_uri) : NULL;
     rdf_li_element = raptor_new_xml_element(rdf_li_qname, NULL, base_uri_copy);
-    attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, sizeof(raptor_qname*));
+    attrs = RAPTOR_CALLOC(raptor_qname**, 1, sizeof(raptor_qname*));
     ruri_string = raptor_uri_to_relative_uri_string(base_uri, item_item->uri);
     attrs[0] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
                                                           rss_serializer->default_nspace,
@@ -1520,11 +1526,8 @@ raptor_rss10_emit_rdfxml_item_triples(raptor_serializer *serializer,
   if(rss_serializer->rss_triples_mode == 2) {
     /* raptor_rdfxmla_serialize_set_write_typed_nodes(ser, 0); */
   }
-  
-  if(base_uri)
-    base_uri = raptor_uri_copy(base_uri);
 
-  /* after this call, ser owns (this copy of) base_uri and does
+  /* after this call, ser does
    * NOT own serializer->iostream and will not destroy it
    * when raptor_free_serializer(ser) is called.
    */
@@ -1605,13 +1608,15 @@ raptor_rss10_ensure_atom_feed_valid(raptor_rss10_serializer_context *rss_seriali
   int i;
   raptor_rss_item* item;
   raptor_rss_model* rss_model;
-  struct timeval tv;
   time_t now = 0;
   int size;
   
 #ifdef HAVE_GETTIMEOFDAY
+  struct timeval tv;
   if(!gettimeofday(&tv, NULL))
     now = tv.tv_sec;
+#else
+  now = time(NULL);
 #endif
   
   is_atom = rss_serializer->is_atom;
@@ -1680,7 +1685,7 @@ raptor_rss10_ensure_atom_feed_valid(raptor_rss10_serializer_context *rss_seriali
     /* atom:title - defaults to "untitled" */
     if(!item->fields[RAPTOR_RSS_FIELD_ATOM_TITLE]) {
       raptor_rss_field* field = raptor_rss_new_field(rss_serializer->world);
-      field->value = (unsigned char*)RAPTOR_MALLOC(cstring, 9);
+      field->value = RAPTOR_MALLOC(unsigned char*, 9);
       memcpy(field->value, "untitled", 9);
       raptor_rss_item_add_field(item, RAPTOR_RSS_FIELD_ATOM_TITLE, field);
     }
@@ -1786,8 +1791,7 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
 
     if(!is_atom && item->uri) {
       unsigned char* ruri_string;
-      attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1,
-                                             sizeof(raptor_qname*));
+      attrs = RAPTOR_CALLOC(raptor_qname**, 1, sizeof(raptor_qname*));
       ruri_string = raptor_uri_to_relative_uri_string(base_uri, item->uri);
       attrs[0] = raptor_new_qname_from_namespace_local_name(serializer->world,
                                                             rss_serializer->default_nspace,
@@ -1850,7 +1854,7 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
           item->fields[f] = NULL;
         } else {
           field = raptor_rss_new_field(serializer->world);
-          field->value = (unsigned char*)RAPTOR_MALLOC(cstring, 8);
+          field->value = RAPTOR_MALLOC(unsigned char*, 8);
           memcpy(field->value, "unknown", 8);
         }
         raptor_rss_item_add_field(author_item, RAPTOR_RSS_FIELD_ATOM_NAME,
@@ -1883,13 +1887,13 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
     for(field = item->fields[f]; field; field = field->next) {
       raptor_xml_element* predicate;
 
-      base_uri_copy = base_uri ? raptor_uri_copy(base_uri) : NULL;
-      predicate = raptor_new_xml_element(raptor_qname_copy(serializer->world->rss_fields_info_qnames[f]), NULL, base_uri_copy);
-
       /* Use atom:summary in preference */
       if(is_atom && f == RAPTOR_RSS_FIELD_DESCRIPTION)
         continue;
 
+      base_uri_copy = base_uri ? raptor_uri_copy(base_uri) : NULL;
+      predicate = raptor_new_xml_element(raptor_qname_copy(serializer->world->rss_fields_info_qnames[f]), NULL, base_uri_copy);
+
       if(is_atom && field->uri) {
         unsigned char* ruri_string;
         size_t len;
@@ -1906,8 +1910,8 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
            !item->fields[RAPTOR_RSS_FIELD_ATOM_CONTENT]) {
           /* atom:link to URI and there is no atom:content */
           raptor_qname **predicate_attrs = NULL;
-          predicate_attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 2, 
-                                                           sizeof(raptor_qname*));
+          predicate_attrs = RAPTOR_CALLOC(raptor_qname**, 2, 
+                                          sizeof(raptor_qname*));
           predicate_attrs[0] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
                                                                           NULL,
                                                                           (const unsigned char*)"href",
@@ -1932,8 +1936,8 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
           else
             content_type = (const unsigned char*)"text/html";
 
-          predicate_attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 2, 
-                                                           sizeof(raptor_qname*));
+          predicate_attrs = RAPTOR_CALLOC(raptor_qname**, 2,
+                                          sizeof(raptor_qname*));
           predicate_attrs[0] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
                                                                           NULL,
                                                                           (const unsigned char*)"src",
@@ -1953,7 +1957,8 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
           raptor_xml_writer_empty_element(xml_writer, predicate);
         } else {
           raptor_xml_writer_start_element(xml_writer, predicate);
-          raptor_xml_writer_cdata_counted(xml_writer, ruri_string, len);
+          raptor_xml_writer_cdata_counted(xml_writer, ruri_string,
+                                          (unsigned int)len);
           raptor_xml_writer_end_element(xml_writer, predicate);
         }
         raptor_free_memory(ruri_string);
@@ -1975,8 +1980,7 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
             int attr_count = 0;
             unsigned char* ruri_string;
 
-            attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 3, 
-                                                   sizeof(raptor_qname*));
+            attrs = RAPTOR_CALLOC(raptor_qname**, 3, sizeof(raptor_qname*));
             ruri_string = raptor_uri_to_relative_uri_string(base_uri, field->uri);
             attrs[attr_count] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
                                                                            rss_serializer->default_nspace,
@@ -2008,8 +2012,7 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
           unsigned char* ruri_string;
 
           /* not an rss:item with an rss:enclosure field */
-          attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, 
-                                                 sizeof(raptor_qname*));
+          attrs = RAPTOR_CALLOC(raptor_qname**, 1, sizeof(raptor_qname*));
           ruri_string = raptor_uri_to_relative_uri_string(base_uri, field->uri);
           attrs[0] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
                                                                 rss_serializer->default_nspace,
@@ -2026,8 +2029,8 @@ raptor_rss10_emit_item(raptor_serializer* serializer,
         
         if(is_xhtml_content && !prefer_cdata) {
           raptor_qname **predicate_attrs = NULL;
-          predicate_attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1,
-                                                           sizeof(raptor_qname*));
+          predicate_attrs = RAPTOR_CALLOC(raptor_qname**, 1,
+                                          sizeof(raptor_qname*));
           if(is_atom)
             predicate_attrs[0] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
                                                                             NULL,
@@ -2190,8 +2193,7 @@ raptor_rss10_serialize_end(raptor_serializer* serializer)
      RAPTOR_OPTIONS_GET_NUMERIC(serializer, RAPTOR_OPTION_WRITE_BASE_URI)) {
     const unsigned char* base_uri_string;
 
-    attrs = (raptor_qname **)RAPTOR_CALLOC(qnamearray, 1, 
-                                           sizeof(raptor_qname*));
+    attrs = RAPTOR_CALLOC(raptor_qname**, 1, sizeof(raptor_qname*));
 
     base_uri_string = raptor_uri_as_string(serializer->base_uri);
     attrs[attrs_count++] = raptor_new_qname_from_namespace_local_name(rss_serializer->world,
@@ -2252,6 +2254,9 @@ raptor_rss10_serialize_end(raptor_serializer* serializer)
   }
 
 
+  if(entry_uri)
+    raptor_free_uri(entry_uri);
+
   raptor_xml_writer_end_element(xml_writer, rss_serializer->root_element);
 
   raptor_free_xml_element(rss_serializer->root_element);
