@@ -44,7 +44,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
@@ -281,10 +281,9 @@ raptor_turtle_emit_blank(raptor_serializer *serializer,
           
   } else {
     /* Blank node that needs an explicit name */
-    const unsigned char *node_id = node->term->value.blank.string;
-
-    raptor_turtle_writer_raw(context->turtle_writer, (const unsigned char*)"_:");
-    raptor_turtle_writer_raw(context->turtle_writer, node_id);
+    raptor_turtle_writer_bnodeid(context->turtle_writer,
+                                 node->term->value.blank.string,
+                                 node->term->value.blank.string_len);
   }
 
   RAPTOR_DEBUG2("Emitted %p\n", node);
@@ -339,7 +338,9 @@ raptor_turtle_emit_subject_list_items(raptor_serializer* serializer,
 
       case RAPTOR_TERM_TYPE_UNKNOWN:
       default:
-        RAPTOR_FATAL1("Unsupported identifier type\n");
+        raptor_log_error_formatted(serializer->world, RAPTOR_LOG_LEVEL_ERROR,
+                                   NULL, "Triple has unsupported term type %d", 
+                                   object->term->type);
         break;
 
     }
@@ -422,10 +423,16 @@ raptor_turtle_emit_subject_collection_items(raptor_serializer* serializer,
 
       case RAPTOR_TERM_TYPE_UNKNOWN:
       default:
-        RAPTOR_FATAL1("Unsupported identifier type\n");
+        raptor_log_error_formatted(serializer->world, RAPTOR_LOG_LEVEL_ERROR,
+                                   NULL, "Triple has unsupported term type %d", 
+                                   object->term->type);
         break;
     }
 
+    /* Return error if emitting something failed above */
+    if(rv)
+      return rv;
+
     /* last item */
     rv = raptor_avltree_iterator_next(iter);
     if(rv)
@@ -562,10 +569,16 @@ raptor_turtle_emit_subject_properties(raptor_serializer* serializer,
           
       case RAPTOR_TERM_TYPE_UNKNOWN:
       default:
-        RAPTOR_FATAL1("Unsupported identifier type\n");
+        raptor_log_error_formatted(serializer->world, RAPTOR_LOG_LEVEL_ERROR,
+                                   NULL, "Triple has unsupported term type %d", 
+                                   object->term->type);
         break;
     }    
     
+    /* Return error if emitting something failed above */
+    if(rv)
+      return rv;
+
     last_predicate = predicate;
   }
 
@@ -651,6 +664,7 @@ raptor_turtle_emit_subject(raptor_serializer *serializer,
     if(rc)
       return rc;
     blank = 0;
+    collection = 0;
     
   } else if(subject->node->term->type == RAPTOR_TERM_TYPE_BLANK) {
     if((subject->node->count_as_subject == 1 && 
@@ -664,7 +678,7 @@ raptor_turtle_emit_subject(raptor_serializer *serializer,
       const unsigned char* genid = subject->node->term->value.blank.string;
       size_t len = strlen((const char*)genid);
       unsigned char* subject_str;
-      subject_str= (unsigned char *)RAPTOR_MALLOC(cstring, len+3);
+      subject_str= RAPTOR_MALLOC(unsigned char*, len + 3);
       if(!subject_str)
         return 1;
 
@@ -672,7 +686,7 @@ raptor_turtle_emit_subject(raptor_serializer *serializer,
       subject_str[1]=':';
       memcpy(&subject_str[2], genid, len + 1);
       raptor_turtle_writer_raw(turtle_writer, subject_str);
-      RAPTOR_FREE(cstring, subject_str);
+      RAPTOR_FREE(char*, subject_str);
     }
   } 
 
@@ -1130,13 +1144,15 @@ raptor_turtle_serialize_finish_factory(raptor_serializer_factory* factory)
 
 static const char* const turtle_names[2] = { "turtle", NULL};
 
-static const char* const turtle_uri_strings[2] = {
+static const char* const turtle_uri_strings[3] = {
+  "http://www.w3.org/ns/formats/Turtle",
   "http://www.dajobe.org/2004/01/turtle/",
   NULL
 };
   
-#define TURTLE_TYPES_COUNT 5
+#define TURTLE_TYPES_COUNT 6
 static const raptor_type_q turtle_types[TURTLE_TYPES_COUNT + 1] = {
+  { "text/turtle", 11, 10},
   { "application/turtle", 18, 10},
   { "application/x-turtle", 20, 8},
   { "text/n3", 7, 3},
@@ -1176,3 +1192,220 @@ raptor_init_serializer_turtle(raptor_world* world)
                                              &raptor_turtle_serializer_register_factory);
 }
 
+
+
+/**
+ * raptor_uri_turtle_write:
+ * @world: world
+ * @iostr: iostream for writing
+ * @uri: uri
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Write #raptor_uri to a stream in turtle syntax (using QNames).
+ *
+ * Return value: non-0 on failure
+ */
+int
+raptor_uri_turtle_write(raptor_world *world,
+                        raptor_iostream* iostr,
+                        raptor_uri* uri, 
+                        raptor_namespace_stack *nstack,
+                        raptor_uri *base_uri)
+{
+  raptor_qname* qname;
+  raptor_turtle_writer* turtle_writer;
+  
+  turtle_writer = raptor_new_turtle_writer(world, base_uri, 0, nstack, iostr);
+  if(!turtle_writer) {
+    return 1;
+  }
+  
+  qname = raptor_new_qname_from_namespace_uri(nstack, uri, 10);
+
+  /* XML Names allow leading '_' and '.' anywhere but Turtle does not */
+  if(qname && !raptor_turtle_is_legal_turtle_qname(qname)) {
+    raptor_free_qname(qname);
+    qname = NULL;
+  }
+
+  if(qname) {
+    raptor_turtle_writer_qname(turtle_writer, qname);
+    
+    raptor_free_qname(qname);
+  } else {
+    raptor_turtle_writer_reference(turtle_writer, uri);
+  }
+  
+  raptor_free_turtle_writer(turtle_writer);
+  
+  return 0;
+}
+
+
+
+/**
+ * raptor_term_turtle_write:
+ * @iostr: iostream for writing
+ * @term: term
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Write #raptor_term to a stream in turtle syntax (using QNames).
+ *
+ * Return value: non-0 on failure
+ */
+int
+raptor_term_turtle_write(raptor_iostream* iostr,
+                         raptor_term* term, 
+                         raptor_namespace_stack *nstack,
+                         raptor_uri *base_uri)
+{
+  int rc = 0;
+  raptor_turtle_writer* turtle_writer;
+  
+  turtle_writer = raptor_new_turtle_writer(term->world, base_uri, 0, nstack, iostr);
+  if(!turtle_writer) {
+    return 1;
+  }
+  
+  if(term->type == RAPTOR_TERM_TYPE_URI) {
+    rc = raptor_uri_turtle_write(term->world, iostr, term->value.uri, nstack, base_uri);
+  } else if(term->type == RAPTOR_TERM_TYPE_LITERAL) {
+    raptor_turtle_writer_literal(turtle_writer, nstack,
+                                 term->value.literal.string,
+                                 term->value.literal.language, 
+                                 term->value.literal.datatype);
+  } else if(term->type == RAPTOR_TERM_TYPE_BLANK) {
+    raptor_bnodeid_ntriples_write(term->value.blank.string, term->value.blank.string_len, iostr);
+  } else {
+    rc = 2;
+  }
+  
+  raptor_free_turtle_writer(turtle_writer);
+  
+  return rc;
+}
+
+
+
+/**
+ * raptor_uri_to_turtle_counted_string:
+ * @world: world
+ * @uri: uri
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ * @len_p: Pointer to location to store length of new string (if not NULL)
+ *
+ * Convert #raptor_uri to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Return value: the new string or NULL on failure.  The length of
+ * the new string is returned in *@len_p if len_p is not NULL.
+ */
+unsigned char*
+raptor_uri_to_turtle_counted_string(raptor_world *world,
+                            raptor_uri* uri, 
+                            raptor_namespace_stack *nstack,
+                            raptor_uri *base_uri,
+                            size_t *len_p)
+{
+  int rc;
+  raptor_iostream* iostr;
+  unsigned char *s;
+  iostr = raptor_new_iostream_to_string(world,
+                                        (void**)&s, len_p, malloc);
+  if(!iostr)
+    return NULL;
+
+  rc = raptor_uri_turtle_write(world, iostr, uri, nstack, base_uri);
+
+  raptor_free_iostream(iostr);
+  if(rc) {
+    free(s);
+    s = NULL;
+  }
+  
+  return s;
+}
+
+/**
+ * raptor_uri_to_turtle_string:
+ * @world: world
+ * @uri: uri
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Convert #raptor_uri to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Return value: the new string or NULL on failure.
+ */
+unsigned char*
+raptor_uri_to_turtle_string(raptor_world *world,
+                            raptor_uri* uri, 
+                            raptor_namespace_stack *nstack,
+                            raptor_uri *base_uri)
+{
+  return raptor_uri_to_turtle_counted_string(world, uri, nstack, base_uri, NULL);
+}
+
+
+
+/**
+ * raptor_term_to_turtle_counted_string:
+ * @term: term
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ * @len_p: Pointer to location to store length of new string (if not NULL)
+ *
+ * Convert #raptor_term to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Return value: the new string or NULL on failure.  The length of
+ * the new string is returned in *@len_p if len_p is not NULL.
+ */
+unsigned char*
+raptor_term_to_turtle_counted_string(raptor_term* term, 
+                             raptor_namespace_stack *nstack,
+                             raptor_uri *base_uri,
+                             size_t *len_p)
+{
+  int rc;
+  raptor_iostream* iostr;
+  unsigned char *s;
+  iostr = raptor_new_iostream_to_string(term->world,
+                                        (void**)&s, len_p, malloc);
+  if(!iostr)
+    return NULL;
+
+  rc = raptor_term_turtle_write(iostr, term, nstack, base_uri);
+
+  raptor_free_iostream(iostr);
+  if(rc) {
+    free(s);
+    s = NULL;
+  }
+  
+  return s;
+}
+
+/**
+ * raptor_term_to_turtle_string:
+ * @term: term
+ * @nstack: namespace stack
+ * @base_uri: base URI
+ *
+ * Convert #raptor_term to a string.
+ * Caller has responsibility to free the string.
+ *
+ * Return value: the new string or NULL on failure.
+ */
+unsigned char*
+raptor_term_to_turtle_string(raptor_term* term, 
+                             raptor_namespace_stack *nstack,
+                             raptor_uri *base_uri)
+{
+  return raptor_term_to_turtle_counted_string(term, nstack, base_uri, NULL);
+}
+
