@@ -42,7 +42,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 
 
@@ -122,7 +122,7 @@ raptor_new_www_with_connection(raptor_world* world, void *connection)
 
   raptor_world_open(world);
 
-  www = (raptor_www* )RAPTOR_CALLOC(www, 1, sizeof(*www));
+  www = RAPTOR_CALLOC(raptor_www*, 1, sizeof(*www));
   if(!www)
     return NULL;
 
@@ -184,27 +184,27 @@ raptor_free_www(raptor_www* www)
   /* free context */
   if(www->type) {
     if(www->free_type)
-      RAPTOR_FREE(cstring, www->type);
+      RAPTOR_FREE(char*, www->type);
     www->type = NULL;
   }
   
   if(www->user_agent) {
-    RAPTOR_FREE(cstring, www->user_agent);
+    RAPTOR_FREE(char*, www->user_agent);
     www->user_agent = NULL;
   }
 
   if(www->cache_control) {
-    RAPTOR_FREE(cstring, www->cache_control);
+    RAPTOR_FREE(char*, www->cache_control);
     www->cache_control = NULL;
   }
 
   if(www->proxy) {
-    RAPTOR_FREE(cstring, www->proxy);
+    RAPTOR_FREE(char*, www->proxy);
     www->proxy = NULL;
   }
 
   if(www->http_accept) {
-    RAPTOR_FREE(cstring, www->http_accept);
+    RAPTOR_FREE(char*, www->http_accept);
     www->http_accept = NULL;
   }
 
@@ -289,7 +289,7 @@ raptor_www_set_user_agent(raptor_www* www, const char *user_agent)
   }
   
   ua_len = strlen(user_agent);
-  ua_copy = (char*)RAPTOR_MALLOC(cstring, ua_len + 1);
+  ua_copy = RAPTOR_MALLOC(char*, ua_len + 1);
   if(!ua_copy)
     return;
 
@@ -318,7 +318,7 @@ raptor_www_set_proxy(raptor_www* www, const char *proxy)
     return;
 
   proxy_len = strlen(proxy);
-  proxy_copy = (char*)RAPTOR_MALLOC(cstring, proxy_len + 1);
+  proxy_copy = RAPTOR_MALLOC(char*, proxy_len + 1);
   if(!proxy_copy)
     return;
 
@@ -348,7 +348,7 @@ raptor_www_set_http_accept(raptor_www* www, const char *value)
     len += 1 + value_len; /* " "+value */
   }
   
-  value_copy = (char*)RAPTOR_MALLOC(cstring, len);
+  value_copy = RAPTOR_MALLOC(char*, len);
   if(!value_copy)
     return;
   www->http_accept = value_copy;
@@ -366,7 +366,7 @@ raptor_www_set_http_accept(raptor_www* www, const char *value)
     *value_copy = '\0';
   }
 
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG2("Using Accept header: '%s'\n", www->http_accept);
 #endif
 }
@@ -410,7 +410,7 @@ raptor_www_set_http_cache_control(raptor_www* www, const char* cache_control)
   RAPTOR_ASSERT((strlen(header) != header_len), "Cache-Control header length is wrong");
 
   if(www->cache_control) {
-    RAPTOR_FREE(cstring, www->cache_control);
+    RAPTOR_FREE(char*, www->cache_control);
     www->cache_control = NULL;
   }
 
@@ -422,7 +422,7 @@ raptor_www_set_http_cache_control(raptor_www* www, const char* cache_control)
   cc_len = strlen(cache_control);
   len = header_len + 1 + cc_len + 1; /* header+" "+cache_control+"\0" */
   
-  cache_control_copy = (char*)RAPTOR_MALLOC(cstring, len);
+  cache_control_copy = RAPTOR_MALLOC(char*, len);
   if(!cache_control_copy)
     return 1;
   
@@ -441,7 +441,7 @@ raptor_www_set_http_cache_control(raptor_www* www, const char* cache_control)
     *cache_control_copy = '\0';
   }
   
-#if RAPTOR_DEBUG > 1
+#if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG2("Using Cache-Control header: '%s'\n", www->cache_control);
 #endif
 
@@ -467,6 +467,59 @@ raptor_www_set_uri_filter(raptor_www* www,
 }
 
 
+/**
+ * raptor_www_set_ssl_cert_options:
+ * @www: WWW object
+ * @cert_filename: SSL client certificate file
+ * @cert_type: SSL client certificate type (default is "PEM")
+ * @cert_passphrase: SSL client certificate password
+ * 
+ * Set SSL client certificate options (where supported)
+ *
+ * Return value: non-0 when setting options is not supported
+ **/
+int
+raptor_www_set_ssl_cert_options(raptor_www* www,
+                                const char* cert_filename,
+                                const char* cert_type,
+                                const char* cert_passphrase)
+{
+#ifdef RAPTOR_WWW_LIBCURL
+  return raptor_www_curl_set_ssl_cert_options(www, cert_filename, cert_type,
+                                              cert_passphrase);
+#else
+  return 1;
+#endif
+}
+
+
+/**
+ * raptor_www_set_ssl_verify_options:
+ * @www: WWW object
+ * @verify_peer: SSL verify peer - non-0 to verify peer SSL certificate (default)
+ * @verify_host: SSL verify host - 0 none, 1 CN match, 2 host match (default). Other values are ignored.
+ * 
+ * Set whether SSL verifies the authenticity of the peer's certificate
+ *
+ * These options correspond to setting the curl
+ * CURLOPT_SSL_VERIFYPEER and CURLOPT_SSL_VERIFYHOST options.
+ *
+ * Return value: non-0 on failure
+ **/
+int
+raptor_www_set_ssl_verify_options(raptor_www* www, int verify_peer,
+                                  int verify_host)
+{
+#ifdef RAPTOR_WWW_LIBCURL
+  return raptor_www_curl_set_ssl_verify_options(www, verify_peer,
+                                                verify_host);
+#else
+  return 1;
+#endif
+}
+
+
+
 /**
  * raptor_www_get_connection:
  * @www: #raptor_www object 
@@ -539,16 +592,14 @@ raptor_www_error(raptor_www* www, const char *message, ...)
 static int 
 raptor_www_file_handle_fetch(raptor_www* www, FILE* fh) 
 {
-  unsigned char buffer[RAPTOR_WWW_BUFFER_SIZE+1];
-  
   while(!feof(fh)) {
-    int len = fread(buffer, 1, RAPTOR_WWW_BUFFER_SIZE, fh);
+    size_t len = fread(www->buffer, 1, RAPTOR_WWW_BUFFER_SIZE, fh);
     if(len > 0) {
       www->total_bytes += len;
-      buffer[len]='\0';
+      www->buffer[len]='\0';
       
       if(www->write_bytes)
-        www->write_bytes(www, www->write_bytes_userdata, buffer, len, 1);
+        www->write_bytes(www, www->write_bytes_userdata, www->buffer, len, 1);
     }
 
     if(feof(fh) || www->failed)
@@ -583,7 +634,7 @@ raptor_www_file_fetch(raptor_www* www)
 #if defined(HAVE_UNISTD_H) && defined(HAVE_SYS_STAT_H)
   if(!stat(filename, &buf) && S_ISDIR(buf.st_mode)) {
     raptor_www_error(www, "Cannot read from a directory '%s'", filename);
-    RAPTOR_FREE(cstring, filename);
+    RAPTOR_FREE(char*, filename);
     www->status_code = 404;
     return 1;
   }
@@ -593,7 +644,7 @@ raptor_www_file_fetch(raptor_www* www)
   if(!fh) {
     raptor_www_error(www, "file '%s' open failed - %s",
                      filename, strerror(errno));
-    RAPTOR_FREE(cstring, filename);
+    RAPTOR_FREE(char*, filename);
     www->status_code = (errno == EACCES) ? 403: 404;
     www->failed = 1;
     
@@ -603,7 +654,7 @@ raptor_www_file_fetch(raptor_www* www)
   raptor_www_file_handle_fetch(www, fh);
   fclose(fh);
 
-  RAPTOR_FREE(cstring, filename);
+  RAPTOR_FREE(char*, filename);
   
   return www->failed;
 }
@@ -629,9 +680,11 @@ raptor_www_fetch(raptor_www *www, raptor_uri *uri)
   www->locator.line= -1;
   www->locator.column= -1;
 
-  if(www->uri_filter)
-    if(www->uri_filter(www->uri_filter_user_data, uri))
-      return status;
+  if(www->uri_filter) {
+    int rc = www->uri_filter(www->uri_filter_user_data, uri);
+    if(rc)
+      return rc;
+  }
   
 #ifdef RAPTOR_WWW_NONE
   status = raptor_www_file_fetch(www);
@@ -672,7 +725,7 @@ raptor_www_fetch_to_string_write_bytes(raptor_www* www, void *userdata,
                                        size_t nmemb)
 {
   raptor_stringbuffer* sb = (raptor_stringbuffer*)userdata;
-  int len = size*nmemb;
+  size_t len = size * nmemb;
 
   raptor_stringbuffer_append_counted_string(sb, (unsigned char*)ptr, len, 1);
 }
